@@ -10,6 +10,9 @@
 /* Define if your system supports binary pipes (i.e. Unix) */
 #undef DRV_PIPE
 
+/* Define if you want a .aiff file writer driver */
+#undef DRV_AIFF
+
 /* Define if the AudioFile driver is compiled */
 #undef DRV_AF
 /* Define if the AIX audio driver is compiled */
@@ -20,8 +23,12 @@
 #undef DRV_ESD
 /* Define if the HP-UX audio driver is compiled */
 #undef DRV_HP
+/* Define if the Network Audio System driver is compiled */
+#undef DRV_NAS
 /* Define if the Open Sound System driver is compiled */
 #undef DRV_OSS
+/* Define if the Linux SAM9407 driver is compiled */
+#undef DRV_SAM9407
 /* Define if the SGI audio driver is compiled */
 #undef DRV_SGI
 /* Define if the Sun audio driver or compatible (NetBSD, OpenBSD)
@@ -29,6 +36,10 @@
 #undef DRV_SUN
 /* Define if the Linux Ultra driver is compiled */
 #undef DRV_ULTRA
+/* Define this if you want the MacOS X CoreAudio driver */
+#undef DRV_OSX
+/* Define this if you want the Carbon Mac Audio driver */
+#undef DRV_MAC
 
 /* Define if you want a debug version of the library */
 #undef MIKMOD_DEBUG
@@ -43,16 +54,15 @@
 #undef SUNOS
 /* Define if your system is AIX 3.* - might be needed for 4.* too */
 #undef AIX
-/* Define if your system has random(3) and srandom(3) */
-#undef HAVE_SRANDOM
-/* Define if your system has snprintf(3) */
-#undef HAVE_SNPRINTF
 /* Define if your system defines random(3) and srandom(3) in math.h instead
    of stdlib.h */
 #undef SRANDOM_IN_MATH_H
-/* Define if your libesd defines esd_close (only present in 0.2.5+) */
-#undef HAVE_ESD_CLOSE
 /* Define if EsounD driver depends on ALSA */
 #undef MIKMOD_DYNAMIC_ESD_NEEDS_ALSA
 /* Define if your system has RTLD_GLOBAL defined in <dlfcn.h> */
 #undef HAVE_RTLD_GLOBAL
+/* Define if your system needs leading underscore to function names in dlsym() calls */
+#undef DLSYM_NEEDS_UNDERSCORE
+
+/* define this if you are running a bigendian system (motorola, sparc, etc) */
+#undef WORDS_BIGENDIAN
