@@ -1,6 +1,6 @@
 /*==============================================================================
 
-  $Id: dl_hpux.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   simple dlopen()-like implementation above HP-UX shl_xxx() API
 
@@ -14,16 +14,14 @@
  * disk is dead and I don't have the system tapes...
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"	/* const */
-#endif
-
-#ifdef MIKMOD_DLAPI_HP
-
 #include <dl.h>
 #include <malloc.h>
 #include <string.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"	/* const */
+#endif
+
 #include "dlfcn.h"
 
 void *dlopen(const char *name, int flags)
@@ -43,7 +41,7 @@ void *dlopen(const char *name, int flags)
 		(flags & RTLD_LAZY ? BIND_DEFERRED : BIND_IMMEDIATE) | DYNAMIC_PATH,
 	    0L);
 
-	free(library);
+	MikMod_free(library);
 
 	return (void *)handle;
 }
@@ -63,5 +61,4 @@ void *dlsym(void *handle, const char *sym)
 	return NULL;
 }
 
-#endif
 /* ex:set ts=4: */
