@@ -6,21 +6,21 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 	02111-1307, USA.
 */
-
+  
 /*==============================================================================
 
-  $Id: drv_aix.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output to AIX series audio device
 
@@ -65,7 +65,7 @@ static void AIX_CommandLine(CHAR *cmdline)
 		if((buf<12)||(buf>19)) buf=BUFFERSIZE;
 		buffersize=1<<buf;
 
-		free(buffer);
+		MikMod_free(buffer);
 	}
 }
 
@@ -81,7 +81,7 @@ static BOOL AIX_IsThere(void)
 	return 1;
 }
 
-static int AIX_Init(void)
+static BOOL AIX_Init(void)
 {
 	audio_init    a_init;
 	audio_control a_control;
@@ -92,24 +92,24 @@ static int AIX_Init(void)
 		_mm_errno=MMERR_16BIT_ONLY;
 		return 1;
 	}
-
+	
 	if((fd=open("/dev/acpa0/1",O_WRONLY))<0)
 		if((fd=open("/dev/paud0/1",O_WRONLY))<0)
 			if((fd=open("/dev/baud0/1",O_WRONLY))<0) {
 				_mm_errno=MMERR_OPENING_AUDIO;
 				return 1;
 			}
-
+	
 	t_info.master_volume=0x7fff;
 	t_info.dither_percent=0;
-
+	
 	a_init.srate=md_mixfreq;
 	a_init.bits_per_sample=16;
 	a_init.channels=(md_mode&DMODE_STEREO)?2:1;
 	a_init.mode=PCM;
 	a_init.flags=FIXED|BIG_ENDIAN|TWOS_COMPLEMENT;
 	a_init.operation=PLAY;
-
+	
 	a_change.balance=0x3fff0000;
 	a_change.balance_delay=0;
 	a_change.volume=(long)((float)1.0*(float)0x7FFFFFFF);
@@ -138,9 +138,9 @@ static int AIX_Init(void)
 		_mm_errno=MMERR_AIX_CONFIG_START;
 		return 1;
 	}
-
-	if (!(audiobuffer=(SBYTE*)_mm_malloc(buffersize))) return 1;
-
+	
+	if (!(audiobuffer=(SBYTE*)MikMod_malloc(buffersize))) return 1;
+	
 	return VC_Init();
 }
 
@@ -151,7 +151,7 @@ static void AIX_Exit(void)
 		close(fd);
 		fd=-1;
 	}
-	_mm_free(audiobuffer);
+	MikMod_free(audiobuffer);
 }
 
 static void AIX_Update(void)
@@ -170,7 +170,7 @@ MIKMODAPI MDRIVER drv_aix={
 	"AIX Audio driver v1.2",
 	0,255,
 	"AIX",
-
+	"buffer:r:12,19,15:Audio buffer log2 size\n",
 	AIX_CommandLine,
 	AIX_IsThere,
 	VC_SampleLoad,
