@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_dart.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output on OS/2 MMPM/2 using direct audio (DART)
 
@@ -72,21 +72,21 @@ static void Dart_CommandLine(CHAR *cmdline)
 		buf = atoi(ptr);
 		if (buf >= 12 && buf <= 16)
 			BufferSize = 1 << buf;
-		free(ptr);
+		MikMod_free(ptr);
 	}
-
+	
 	if ((ptr = MD_GetAtom("count", cmdline, 0))) {
 		buf = atoi(ptr);
 		if (buf >= 2 && buf <= MAX_BUFFERCOUNT)
 			BufferCount = buf;
-		free(ptr);
+		MikMod_free(ptr);
 	}
-
+	
 	if ((ptr = MD_GetAtom("device", cmdline, 0))) {
 		buf = atoi(ptr);
 		if (buf >= 0 && buf <= 8)
 			DeviceIndex = buf;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 }
 
@@ -99,7 +99,7 @@ static LONG APIENTRY Dart_UpdateBuffers(ULONG ulStatus, PMCI_MIX_BUFFER pBuffer,
 	/* sanity check */
 	if (!pBuffer)
 		return TRUE;
-
+	
 	/* if we have finished a buffer, we're ready to play a new one */
 	if ((ulFlags == MIX_WRITE_COMPLETE) ||
 		((ulFlags == (MIX_WRITE_COMPLETE | MIX_STREAM_ERROR)) &&
@@ -136,7 +136,7 @@ static BOOL Dart_IsPresent(void)
 	return 1;
 }
 
-static int Dart_Init(void)
+static BOOL Dart_Init(void)
 {
 	MCI_AMP_OPEN_PARMS AmpOpenParms;
 	MCI_GENERIC_PARMS GenericParms;
@@ -182,7 +182,7 @@ static int Dart_Init(void)
 		/* DART suggested buffer size is somewhat too big. We compute a size
 		   for circa 1/4" of playback. */
 		int bit;
-
+		
 		BufferSize = md_mixfreq >> 2;
 		if (md_mode & DMODE_STEREO)
 			BufferSize <<= 1;
@@ -229,7 +229,7 @@ static void Dart_Exit(void)
 	}
 }
 
-static int Dart_PlayStart(void)
+static BOOL Dart_PlayStart(void)
 {
 	MCI_GENERIC_PARMS GenericParms;
 	int i;
@@ -279,7 +279,9 @@ MIKMODAPI MDRIVER drv_dart = {
 	"OS/2 DART driver v1.2",
 	0, 255,
 	"dart",
-
+	"device:r:0,8,0:Waveaudio device index to use (0 - default)\n"
+        "buffer:r:12,16:Audio buffer log2 size\n"
+        "count:r:2,8,2:Number of audio buffers\n",
 	Dart_CommandLine,
 	Dart_IsPresent,
 	VC_SampleLoad,
