@@ -6,21 +6,21 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 	02111-1307, USA.
 */
-
+  
 /*==============================================================================
 
-  $Id: drv_hp.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output to HP 9000 series /dev/audio
 
@@ -66,12 +66,12 @@ static void HP_CommandLine(CHAR *cmdline)
 		if((buf<12)||(buf>19)) buf=BUFFERSIZE;
 		buffersize=1<<buf;
 
-		free(buffer);
+		MikMod_free(buffer);
 	}
 
 	if((buffer=MD_GetAtom("headphone",cmdline,1))) {
 		headphone=1;
-		free(buffer);
+		MikMod_free(buffer);
 	} else
 		headphone=0;
 }
@@ -87,10 +87,10 @@ static BOOL HP_IsThere(void)
 	return (errno==EACCES?1:0);
 }
 
-static int HP_Init(void)
+static BOOL HP_Init(void)
 {
 	int flags;
-
+	
 	if (!(md_mode&DMODE_16BITS)) {
 		_mm_errno=MMERR_16BIT_ONLY;
 		return 1;
@@ -110,24 +110,24 @@ static int HP_Init(void)
 		_mm_errno=MMERR_NON_BLOCK;
 		return 1;
 	}
-
+	
 	if (ioctl(fd,AUDIO_SET_DATA_FORMAT,AUDIO_FORMAT_LINEAR16BIT)) {
 		_mm_errno=MMERR_HP_SETSAMPLESIZE;
 		return 1;
 	}
-
+	
 	if (ioctl(fd,AUDIO_SET_SAMPLE_RATE,md_mixfreq)) {
 		_mm_errno=MMERR_HP_SETSPEED;
 		return 1;
 	}
-
+	
 	if (ioctl(fd,AUDIO_SET_CHANNELS,(md_mode&DMODE_STEREO)?2:1)) {
 		_mm_errno=MMERR_HP_CHANNELS;
 		return 1;
 	}
-
+	
 	if (ioctl(fd,AUDIO_SET_OUTPUT,
-		     headphone?AUDIO_OUT_HEADPHONE:AUDIO_OUT_SPEAKER)) {
+	             headphone?AUDIO_OUT_HEADPHONE:AUDIO_OUT_SPEAKER)) {
 		_mm_errno=MMERR_HP_AUDIO_OUTPUT;
 		return 1;
 	}
@@ -137,8 +137,8 @@ static int HP_Init(void)
 		return 1;
 	}
 
-	if (!(audiobuffer=(SBYTE*)_mm_malloc(buffersize))) return 1;
-
+	if (!(audiobuffer=(SBYTE*)MikMod_malloc(buffersize))) return 1;
+	
 	return VC_Init();
 }
 
@@ -151,7 +151,7 @@ static void HP_Exit(void)
 		fd=-1;
 	}
 	if (audiobuffer) {
-		free(audiobuffer);
+		MikMod_free(audiobuffer);
 		audiobuffer=NULL;
 	}
 }
@@ -167,7 +167,8 @@ MIKMODAPI MDRIVER drv_hp={
 	"HP-UX Audio driver v1.3",
 	0,255,
 	"hp",
-
+	"buffer:r:12,19,15:Audio buffer log2 size\n"
+        "headphone:b:0:Use headphone\n",
 	HP_CommandLine,
 	HP_IsThere,
 	VC_SampleLoad,
@@ -198,8 +199,7 @@ MIKMODAPI MDRIVER drv_hp={
 #else
 
 MISSING(drv_hp);
-
+		
 #endif
 
 /* ex:set ts=4: */
-
