@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_nos.c,v 1.1.1.1 2004/01/21 01:36:35 raph Exp $
+  $Id$
 
   Driver for no output
 
@@ -53,14 +53,14 @@ static BOOL NS_IsThere(void)
 
 static BOOL NS_Init(void)
 {
-	zerobuf=(SBYTE*)_mm_malloc(ZEROLEN);
+	zerobuf=(SBYTE*)MikMod_malloc(ZEROLEN);
 	return VC_Init();
 }
 
 static void NS_Exit(void)
 {
 	VC_Exit();
-	_mm_free(zerobuf);
+	MikMod_free(zerobuf);
 }
 
 static void NS_Update(void)
@@ -75,7 +75,7 @@ MIKMODAPI MDRIVER drv_nos={
 	"Nosound Driver v3.0",
 	255,255,
 	"nosound",
-
+	NULL,
 	NULL,
 	NS_IsThere,
 	VC_SampleLoad,
