@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_os2.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output on OS/2 using MMPM/2 MCI interface
 
@@ -33,12 +33,6 @@
 
 */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef DRV_OS2
-
 #define INCL_DOS
 #define INCL_OS2MM
 #include <os2.h>
@@ -48,8 +42,6 @@
 #undef PPFN
 
 #include <stdlib.h>
-#include <string.h>
-#include <process.h> /* _beginthread */
 
 #include "mikmod_internals.h"
 
@@ -125,13 +117,13 @@ static void OS2_CommandLine(CHAR *cmdline)
 		buf = atoi(ptr);
 		if (buf >= 12 && buf <= 16)
 			BufferSize = 1 << buf;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 	if ((ptr = MD_GetAtom("device", cmdline, 0))) {
 		buf = atoi(ptr);
 		if (buf >= 0 && buf <= 8)
 			DeviceIndex = buf;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 }
 
@@ -156,7 +148,7 @@ static BOOL OS2_IsPresent(void)
 	return 1;
 }
 
-static int OS2_Init(void)
+static BOOL OS2_Init(void)
 {
 	MCI_OPEN_PARMS mciOpenParms;
 	MCI_WAVE_SET_PARMS mciWaveSetParms;
@@ -198,7 +190,7 @@ static int OS2_Init(void)
 	   sizes (16K for 44KHz, 16 bit stereo). */
 
 	/* Allocate buffer */
-	if (!(AudioBuffer = _mm_malloc(BufferSize * FRAGMENTS))) {
+	if (!(AudioBuffer = MikMod_malloc(BufferSize * FRAGMENTS))) {
 		_mm_errno = MMERR_OUT_OF_MEMORY;
 		return 1;
 	}
@@ -299,10 +291,10 @@ static void OS2_Exit(void)
 					   (PVOID) & mciGenericParms, 0);
 		DeviceID = 0;
 	}
-	_mm_free(AudioBuffer);
+	MikMod_free(AudioBuffer);
 }
 
-static int OS2_PlayStart(void)
+static BOOL OS2_PlayStart(void)
 {
 	MCI_PLAY_PARMS mciPlayParms;
 	int i;
@@ -350,7 +342,8 @@ MIKMODAPI MDRIVER drv_os2 = {
 	"OS/2 MMPM/2 MCI driver v1.2",
 	0,255,
 	"os2",
-
+	"device:r:0,8,0:Waveaudio device index to use (0 - default)\n"
+        "buffer:r:12,16:Audio buffer log2 size\n",
 	OS2_CommandLine,
 	OS2_IsPresent,
 	VC_SampleLoad,
@@ -378,10 +371,4 @@ MIKMODAPI MDRIVER drv_os2 = {
 	VC_VoiceRealVolume
 };
 
-#else
-
-#include "mikmod_internals.h"
-MISSING(drv_os2);
-
-#endif
 /* ex:set ts=4: */
