@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,9 +20,9 @@
 
 /*==============================================================================
 
-  $Id: drv_oss.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
-  Driver for output on Linux and FreeBSD Open Sound System (OSS) (/dev/dsp)
+  Driver for output on Linux and FreeBSD Open Sound System (OSS) (/dev/dsp) 
 
 ==============================================================================*/
 
@@ -114,25 +114,25 @@ static void OSS_CommandLine(CHAR *cmdline)
 	if((ptr=MD_GetAtom("buffer",cmdline,0))) {
 		fragsize=atoi(ptr);
 		if((fragsize<7)||(fragsize>17)) fragsize=DEFAULT_FRAGSIZE;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 	if((ptr=MD_GetAtom("count",cmdline,0))) {
 		numfrags=atoi(ptr);
 		if((numfrags<2)||(numfrags>255)) numfrags=DEFAULT_NUMFRAGS;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 #endif
 	if((ptr=MD_GetAtom("card",cmdline,0))) {
 		card = atoi(ptr);
 		if((card<0)||(card>99)) card=DEFAULT_CARD;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 }
 
 static char *OSS_GetDeviceName(void)
 {
 	static char sounddevice[20];
-
+	
 	/* First test for devfs enabled Linux sound devices */
 	if (card)
 		sprintf(sounddevice,"/dev/sound/dsp%d",card);
@@ -149,7 +149,7 @@ static char *OSS_GetDeviceName(void)
 		if(access("/dev/dsp0",F_OK))
 			strcpy(sounddevice,"/dev/dsp");
 	}
-
+	
 	return sounddevice;
 }
 
@@ -167,11 +167,11 @@ static BOOL OSS_IsThere(void)
 	return (errno==EACCES?1:0);
 }
 
-static int OSS_Init_internal(void)
+static BOOL OSS_Init_internal(void)
 {
 	int play_stereo,play_rate;
 	int orig_precision,orig_stereo;
-	int formats;
+	long formats;
 #if SOUND_VERSION >= 301
 	audio_buf_info buffinf;
 #endif
@@ -245,20 +245,20 @@ static int OSS_Init_internal(void)
 	/* This call fails on Linux/PPC */
 	if((ioctl(sndfd,SNDCTL_DSP_GETOSPACE,&buffinf)<0))
 		ioctl(sndfd,SNDCTL_DSP_GETBLKSIZE,&buffinf.fragsize);
-	if(!(audiobuffer=(SBYTE*)_mm_malloc(buffinf.fragsize)))
+	if(!(audiobuffer=(SBYTE*)MikMod_malloc(buffinf.fragsize)))
 		return 1;
-
+	
 	buffersize = buffinf.fragsize;
 #else
 	ioctl(sndfd,SNDCTL_DSP_GETBLKSIZE,&buffersize);
-	if(!(audiobuffer=(SBYTE*)_mm_malloc(buffersize)))
+	if(!(audiobuffer=(SBYTE*)MikMod_malloc(buffersize)))
 		return 1;
 #endif
 
 	return VC_Init();
 }
 
-static int OSS_Init(void)
+static BOOL OSS_Init(void)
 {
 #ifdef SNDCTL_DSP_SETFRAGMENT
 	int fragmentsize;
@@ -280,7 +280,7 @@ static int OSS_Init(void)
 	}
 
 	fragmentsize=(numfrags<<16)|fragsize;
-
+	
 	if(ioctl(sndfd,SNDCTL_DSP_SETFRAGMENT,&fragmentsize)<0) {
 		_mm_errno=MMERR_OSS_SETFRAGMENT;
 		return 1;
@@ -293,7 +293,7 @@ static int OSS_Init(void)
 static void OSS_Exit_internal(void)
 {
 	VC_Exit();
-	_mm_free(audiobuffer);
+	MikMod_free(audiobuffer);
 }
 
 static void OSS_Exit(void)
@@ -310,7 +310,7 @@ static void OSS_PlayStop(void)
 {
 	VC_PlayStop();
 
-	ioctl(sndfd,SNDCTL_DSP_POST,0);
+	ioctl(sndfd,SNDCTL_DSP_POST);
 }
 
 static void OSS_Update(void)
@@ -333,7 +333,7 @@ static void OSS_Update(void)
 						   buffinf.bytes:buffinf.fragsize);
 #ifdef AFMT_MU_LAW
 		if (play_precision==AFMT_MU_LAW)
-			unsignedtoulaw((char *)audiobuffer,done);
+			unsignedtoulaw(audiobuffer,done);
 #endif
 		write(sndfd,audiobuffer,done);
 	}
@@ -347,10 +347,10 @@ static void OSS_Update(void)
 #endif
 }
 
-static int OSS_Reset(void)
+static BOOL OSS_Reset(void)
 {
 	OSS_Exit_internal();
-	ioctl(sndfd,SNDCTL_DSP_RESET,0);
+	ioctl(sndfd,SNDCTL_DSP_RESET);
 	return OSS_Init_internal();
 }
 
@@ -360,7 +360,11 @@ MIKMODAPI MDRIVER drv_oss={
 	"Open Sound System driver v1.7",
 	0,255,
 	"oss",
-
+#ifdef SNDCTL_DSP_SETFRAGMENT
+	"buffer:r:7,17,14:Audio buffer log2 size\n"
+	"count:r:2,255,16:Audio buffer count\n"
+#endif	
+	"card:r:0,99,0:Sound card id\n",
 	OSS_CommandLine,
 	OSS_IsThere,
 	VC_SampleLoad,
