@@ -19,11 +19,11 @@
 */
 
 /*==============================================================================
-
-  $Id: drv_sam9407.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
-
+  
+  $Id$
+  
   Driver for the Linux sam9407 driver
-
+  
 ==============================================================================*/
 
 /*
@@ -106,7 +106,7 @@ static void commandLine(CHAR *cmdline)
 
 	if((ptr=MD_GetAtom("card", cmdline, 0))) {
 		card=atoi(ptr);
-		free(ptr);
+		MikMod_free(ptr);
 	}
 }
 
@@ -139,7 +139,7 @@ static SWORD sampleLoad(SAMPLOAD *s, int type)
 	SamModSamples modSamples;
 	SWORD handle;
 	SWORD *samples;
-	int rc;
+int rc;
 
 	for(handle=0; handle<SAM_NUM_BANKS; handle++)
 		if(!banks[handle].inUse)
@@ -161,14 +161,14 @@ static SWORD sampleLoad(SAMPLOAD *s, int type)
 	bankP->loopend=s->sample->loopend ? s->sample->loopend : s->sample->length;
 	bankP->flags=s->sample->flags;
 
-	if(!(samples=(SWORD *)malloc((bankP->length+NO_LOOP_SAMPLES)<<1))) {
+	if(!(samples=(SWORD *)MikMod_malloc((bankP->length+NO_LOOP_SAMPLES)<<1))) {
 		bankP->inUse=0;
 		_mm_errno=MMERR_SAMPLE_TOO_BIG;
 		return -1;
 	}
 
 	if(SL_Load(samples, s, bankP->length)) {
-		free(samples);
+		MikMod_free(samples);
 		bankP->inUse=0;
 		_mm_errno=MMERR_SAMPLE_TOO_BIG;
 		return -1;
@@ -182,16 +182,16 @@ static SWORD sampleLoad(SAMPLOAD *s, int type)
 	}
 
 	modSamples.handle=handle;
-	modSamples.data=(unsigned short *)samples;
+	modSamples.data=samples;
 	modSamples.size=bankP->length;
 	if((rc=ioctl(modfd, SAM_IOC_MOD_SAMPLES_LOAD, &modSamples))<0) {
-		free(samples);
+		MikMod_free(samples);
 		bankP->inUse=0;
 		_mm_errno=MMERR_SAMPLE_TOO_BIG;
 		return -1;
 	}
 
-	free(samples);
+	MikMod_free(samples);
 
 	return handle;
 }
@@ -216,7 +216,7 @@ static ULONG realSampleLength(int type, SAMPLE *s)
 	return s->length<<1;
 }
 
-static int init(void)
+static BOOL init(void)
 {
 	int i;
 	Voice *voiceP;
@@ -255,7 +255,7 @@ static void exitHook(void)
 	}
 }
 
-static int reset(void)
+static BOOL reset(void)
 {
 	if(ioctl(modfd, SAM_IOC_MOD_RESET)<0)
 		return 1;
@@ -263,12 +263,12 @@ static int reset(void)
 	return 0;
 }
 
-static int setNumVoices(void)
+static BOOL setNumVoices(void)
 {
 	return 0;
 }
 
-static int playStart(void)
+static BOOL playStart(void)
 {
 	ioctl(modfd, SAM_IOC_MOD_TIMER_START);
 	return 0;
@@ -457,7 +457,10 @@ static void voiceStop(UBYTE voice)
 
 static BOOL voiceStopped(UBYTE voice)
 {
-	return (voice<SAM_NUM_VOICES)? !voices[voice].playing : 1;
+	if(voice>=SAM_NUM_VOICES)
+		return 1;
+
+	return voice<SAM_NUM_VOICES ? !voices[voice].playing : 1;
 }
 
 static SLONG voiceGetPosition(UBYTE voice)
@@ -476,6 +479,7 @@ MIKMODAPI MDRIVER drv_sam9407={
 	"Linux sam9407 driver v1.0",
 	SAM_NUM_VOICES, 0,
 	"sam9407",
+
 	commandLine,
 	isPresent,
 	sampleLoad,
