@@ -1,5 +1,5 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
+	(c) 1998, 1999, 2000 Miodrag Vallat and others - see file AUTHORS for
 	complete list.
 
 	This library is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_stdout.c,v 1.25 1999/06/28 03:00:05 miod Exp $
+  $Id$
 
   Output data to stdout
 
@@ -35,7 +35,7 @@
 #endif
 #include <stdlib.h>
 
-#include <mikmod_internals.h>
+#include "mikmod_internals.h"
 
 #define BUFFERSIZE 32768
 
@@ -49,7 +49,7 @@ static BOOL stdout_IsThere(void)
 
 static BOOL stdout_Init(void)
 {
-	if(!(audiobuffer=(SBYTE*)_mm_malloc(BUFFERSIZE))) return 1;
+	if(!(audiobuffer=(SBYTE*)MikMod_malloc(BUFFERSIZE))) return 1;
 #ifdef __EMX__
 	_fsetmode(stdout,"b");
 #endif
@@ -63,7 +63,7 @@ static void stdout_Exit(void)
 	_fsetmode(stdout,"t");
 #endif
 	if (audiobuffer) {
-		free(audiobuffer);
+		MikMod_free(audiobuffer);
 		audiobuffer=NULL;
 	}
 }
@@ -84,13 +84,13 @@ static BOOL stdout_Reset(void)
 	return VC_Init();
 }
 
-MDRIVER drv_stdout={
+MIKMODAPI MDRIVER drv_stdout={
 	NULL,
 	"stdout",
 	"Standard output driver v1.1",
 	0,255,
 	"stdout",
-
+	NULL,
 	NULL,
 	stdout_IsThere,
 	VC_SampleLoad,
