@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_sun.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output on the Sun audio device (/dev/audio).
   Also works under NetBSD and OpenBSD
@@ -180,15 +180,15 @@ static void Sun_CommandLine(CHAR *cmdline)
 		if (buf >= 7 && buf <= 17)
 			fragsize = 1 << buf;
 
-		free(ptr);
+		MikMod_free(ptr);
 	}
 
 	if ((ptr = MD_GetAtom("headphone", cmdline, 1))) {
 		port = AUDIO_HEADPHONE;
-		free(ptr);
+		MikMod_free(ptr);
 	} else if ((ptr = MD_GetAtom("speaker", cmdline, 1))) {
 		port = AUDIO_SPEAKER;
-		free(ptr);
+		MikMod_free(ptr);
 	}
 }
 
@@ -209,7 +209,7 @@ static BOOL Sun_IsThere(void)
 	return 0;
 }
 
-static int Sun_Init(void)
+static BOOL Sun_Init(void)
 {
 	int play_stereo, play_rate;
 #ifdef SUNOS4
@@ -233,7 +233,7 @@ static int Sun_Init(void)
 		return 1;
 	}
 
-	if (!(audiobuffer = (SBYTE *)_mm_malloc(fragsize)))
+	if (!(audiobuffer = (SBYTE *)MikMod_malloc(fragsize)))
 		return 1;
 
 	play_precision = (md_mode & DMODE_16BITS) ? 16 : 8;
@@ -314,7 +314,7 @@ static int Sun_Init(void)
 			play_encoding = AUDIO_ENCODING_ULAW;
 		}
 		if ((!strcmp(audiotype.name, "Am78C201")) ||
-			(!strcmp(audiotype.name, "UltraSound"))
+			(!strcmp(audiotype.name, "UltraSound")) 
 		   ) {
 			/* Gravis UltraSound, AMD Interwave and compatible cards */
 			/* 16bit stereo linear 44kHz */
@@ -336,8 +336,9 @@ static int Sun_Init(void)
 #ifdef SUNOS4
 			play_encoding = AUDIO_ENCODING_LINEAR;
 #else
-			play_encoding = (play_precision == 16) ?
-				 AUDIO_ENCODING_SLINEAR : AUDIO_ENCODING_ULINEAR;
+			play_encoding =
+				(play_precision ==
+				 16) ? AUDIO_ENCODING_SLINEAR : AUDIO_ENCODING_ULINEAR;
 #endif
 	}
 
@@ -404,7 +405,7 @@ static int Sun_Init(void)
 static void Sun_Exit(void)
 {
 	VC_Exit();
-	_mm_free(audiobuffer);
+	MikMod_free(audiobuffer);
 	if (sndfd >= 0) {
 		close(sndfd);
 		sndfd = -1;
@@ -429,7 +430,7 @@ static void Sun_Pause(void)
 	write(sndfd, audiobuffer, done);
 }
 
-static int Sun_PlayStart(void)
+static BOOL Sun_PlayStart(void)
 {
 	struct audio_info audioinfo;
 
@@ -471,6 +472,12 @@ MIKMODAPI MDRIVER drv_sun = {
 #endif
 	0, 255,
 	"audio",
+        "buffer:r:7,17,12:Audio buffer log2 size\n"
+#if defined(SUNOS) || defined(SOLARIS)
+        "headphone:b:0:Use headphone\n"
+        "speaker:b:0:Use speaker\n"
+#endif
+	,
 
 	Sun_CommandLine,
 	Sun_IsThere,
