@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: drv_wav.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Driver for output to a file called MUSIC.WAV
 
@@ -32,8 +32,6 @@
 
 #include "mikmod_internals.h"
 
-#ifdef DRV_WAV
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -55,20 +53,17 @@ static	CHAR *filename=NULL;
 
 static void putheader(void)
 {
-	ULONG rflen = 36;
-	rflen += dumpsize;
-
 	_mm_fseek(wavout,0,SEEK_SET);
 	_mm_write_string("RIFF",wavout);
-	_mm_write_I_ULONG(rflen,wavout);
+	_mm_write_I_ULONG(dumpsize+44-8,wavout);	/* <AWE> must be smaller by 8! */
 	_mm_write_string("WAVEfmt ",wavout);
 	_mm_write_I_ULONG(16,wavout);	/* length of this RIFF block crap */
 
-	_mm_write_I_UWORD(1, wavout);	/* WAVE_FORMAT_PCM :1 */
+	_mm_write_I_UWORD(1, wavout);	/* microsoft format type */
 	_mm_write_I_UWORD((md_mode&DMODE_STEREO)?2:1,wavout);
 	_mm_write_I_ULONG(md_mixfreq,wavout);
-	_mm_write_I_ULONG(md_mixfreq *	((md_mode&DMODE_STEREO)?2:1) *
-					((md_mode&DMODE_16BITS)?2:1), wavout);
+	_mm_write_I_ULONG(md_mixfreq*((md_mode&DMODE_STEREO)?2:1)*
+	                  ((md_mode&DMODE_16BITS)?2:1),wavout);
 	/* block alignment (8/16 bit) */
 	_mm_write_I_UWORD(((md_mode&DMODE_16BITS)?2:1)* 
 	                  ((md_mode&DMODE_STEREO)?2:1),wavout);
@@ -82,7 +77,7 @@ static void WAV_CommandLine(CHAR *cmdline)
 	CHAR *ptr=MD_GetAtom("file",cmdline,0);
 
 	if(ptr) {
-		_mm_free(filename);
+		MikMod_free(filename);
 		filename=ptr;
 	}
 }
@@ -92,9 +87,9 @@ static BOOL WAV_IsThere(void)
 	return 1;
 }
 
-static int WAV_Init(void)
+static BOOL WAV_Init(void)
 {
-#if (MIKMOD_UNIX)
+#if defined unix || (defined __APPLE__ && defined __MACH__)
 	if (!MD_Access(filename?filename:FILENAME)) {
 		_mm_errno=MMERR_OPENING_FILE;
 		return 1;
@@ -110,7 +105,7 @@ static int WAV_Init(void)
 		wavfile=NULL;
 		return 1;
 	}
-	if(!(audiobuffer=(SBYTE*)_mm_malloc(BUFFERSIZE))) {
+	if(!(audiobuffer=(SBYTE*)MikMod_malloc(BUFFERSIZE))) {
 		_mm_delete_file_writer(wavout);
 		fclose(wavfile);unlink(filename?filename:FILENAME);
 		wavfile=NULL;wavout=NULL;
@@ -143,7 +138,7 @@ static void WAV_Exit(void)
 		wavfile=NULL;wavout=NULL;
 	}
 	if(audiobuffer) {
-		free(audiobuffer);
+		MikMod_free(audiobuffer);
 		audiobuffer=NULL;
 	}
 }
@@ -154,14 +149,12 @@ static void WAV_Update(void)
 
 	done=VC_WriteBytes(audiobuffer,BUFFERSIZE);
 
-	if (md_mode & DMODE_16BITS) {
-	/* <AWE> Fix for 16bit samples on big endian systems: Just swap
-	 * bytes via "_mm_write_I_UWORDS ()" if we have 16 bit output.	*/
-		_mm_write_I_UWORDS((UWORD *) audiobuffer,done>>1,wavout);
-	}
-	else {
-		_mm_write_UBYTES(audiobuffer,done,wavout);
-	}
+/*    <AWE> Fix for 16bit samples on big endian systems:				*/
+/*	      Just swap bytes via "_mm_write_I_UWORDS ()" if we have 16 bit output.	*/
+        if (md_mode & DMODE_16BITS)		
+            _mm_write_I_UWORDS((UWORD *) audiobuffer,done>>1,wavout);       
+        else
+            _mm_write_UBYTES(audiobuffer,done,wavout);
 	dumpsize+=done;
 }
 
@@ -171,7 +164,7 @@ MIKMODAPI MDRIVER drv_wav={
 	"Wav disk writer (music.wav) v1.2",
 	0,255,
 	"wav",
-
+	"file:t:music.wav:Output file name\n",
 	WAV_CommandLine,
 	WAV_IsThere,
 	VC_SampleLoad,
@@ -199,10 +192,4 @@ MIKMODAPI MDRIVER drv_wav={
 	VC_VoiceRealVolume
 };
 
-#else
-
-MISSING(drv_wav);
-
-#endif
-
 /* ex:set ts=4: */
