@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_dsm.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   DSIK internal format (DSM) module loader
 
@@ -105,7 +105,7 @@ static	unsigned char DSMSIG[4+4]={'R','I','F','F','D','S','M','F'};
 
 /*========== Loader code */
 
-static BOOL DSM_Test(void)
+BOOL DSM_Test(void)
 {
 	UBYTE id[12];
 
@@ -115,17 +115,17 @@ static BOOL DSM_Test(void)
 	return 0;
 }
 
-static BOOL DSM_Init(void)
+BOOL DSM_Init(void)
 {
-	if(!(dsmbuf=(DSMNOTE *)_mm_malloc(DSM_MAXCHAN*64*sizeof(DSMNOTE)))) return 0;
-	if(!(mh=(DSMSONG *)_mm_calloc(1,sizeof(DSMSONG)))) return 0;
+	if(!(dsmbuf=(DSMNOTE *)MikMod_malloc(DSM_MAXCHAN*64*sizeof(DSMNOTE)))) return 0;
+	if(!(mh=(DSMSONG *)MikMod_calloc(1,sizeof(DSMSONG)))) return 0;
 	return 1;
 }
 
-static void DSM_Cleanup(void)
+void DSM_Cleanup(void)
 {
-	_mm_free(dsmbuf);
-	_mm_free(mh);
+	MikMod_free(dsmbuf);
+	MikMod_free(mh);
 }
 
 static BOOL GetBlockHeader(void)
@@ -133,7 +133,7 @@ static BOOL GetBlockHeader(void)
 	/* make sure we're at the right position for reading the
 	   next riff block, no matter how many bytes read */
 	_mm_fseek(modreader, blocklp+blockln, SEEK_SET);
-
+   
 	while(1) {
 		_mm_read_UBYTES(blockid,4,modreader);
 		blockln=_mm_read_I_ULONG(modreader);
@@ -231,7 +231,7 @@ static UBYTE *DSM_ConvertTrack(DSMNOTE *tr)
 	return UniDup();
 }
 
-static BOOL DSM_Load(BOOL curious)
+BOOL DSM_Load(BOOL curious)
 {
 	int t;
 	DSMINST s;
@@ -265,7 +265,7 @@ static BOOL DSM_Load(BOOL curious)
 	/* set module variables */
 	of.initspeed=mh->speed;
 	of.inittempo=mh->bpm;
-	of.modtype=strdup(DSM_Version);
+	of.modtype=StrDup(DSM_Version);
 	of.numchn=mh->numtrk;
 	of.numpat=mh->numpat;
 	of.numtrk=of.numchn*of.numpat;
@@ -337,13 +337,13 @@ static BOOL DSM_Load(BOOL curious)
 	return 1;
 }
 
-static CHAR *DSM_LoadTitle(void)
+CHAR *DSM_LoadTitle(void)
 {
 	CHAR s[28];
 
 	_mm_fseek(modreader,12,SEEK_SET);
 	if(!_mm_read_UBYTES(s,28,modreader)) return NULL;
-
+   
 	return(DupStr(s,28,1));
 }
 
