@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_gdm.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   General DigiMusic (GDM) module loader
 
@@ -114,9 +114,9 @@ typedef struct GDMSAMPLE {
 static GDMHEADER *mh=NULL;	/* pointer to GDM header */
 static GDMNOTE *gdmbuf=NULL;	/* pointer to a complete GDM pattern */
 
-static CHAR GDM_Version[]="General DigiMusic 1.xx";
+CHAR GDM_Version[]="General DigiMusic 1.xx";
 
-static BOOL GDM_Test(void)
+BOOL GDM_Test(void)
 {
 	/* test for gdm magic numbers */
 	UBYTE id[4];
@@ -134,21 +134,21 @@ static BOOL GDM_Test(void)
 	return 0;
 }
 
-static BOOL GDM_Init(void)
+BOOL GDM_Init(void)
 {
-	if (!(gdmbuf=(GDMNOTE*)_mm_malloc(32*64*sizeof(GDMNOTE)))) return 0;
-	if (!(mh=(GDMHEADER*)_mm_malloc(sizeof(GDMHEADER)))) return 0;
+	if (!(gdmbuf=(GDMNOTE*)MikMod_malloc(32*64*sizeof(GDMNOTE)))) return 0;
+	if (!(mh=(GDMHEADER*)MikMod_malloc(sizeof(GDMHEADER)))) return 0;
 
 	return 1;
 }
 
-static void GDM_Cleanup(void)
+void GDM_Cleanup(void)
 {
-	_mm_free(mh);
-	_mm_free(gdmbuf);
+	MikMod_free(mh);
+	MikMod_free(gdmbuf);
 }
 
-static BOOL GDM_ReadPattern(void)
+BOOL GDM_ReadPattern(void)
 {
 	int pos,flag,ch,i,maxch;
 	GDMNOTE n;
@@ -200,7 +200,7 @@ static BOOL GDM_ReadPattern(void)
 	return 1;
 }
 
-static UBYTE *GDM_ConvertTrack(GDMNOTE*tr)
+UBYTE *GDM_ConvertTrack(GDMNOTE*tr)
 {
 	int t,i=0;
 	UBYTE note,ins,inf;
@@ -337,7 +337,7 @@ static UBYTE *GDM_ConvertTrack(GDMNOTE*tr)
 	return UniDup();
 }
 
-static BOOL GDM_Load(BOOL curious)
+BOOL GDM_Load(BOOL curious)
 {
 	int i,x,u,track;
 	SAMPLE *q;
@@ -389,7 +389,7 @@ static BOOL GDM_Load(BOOL curious)
 	}
 
 	/* now we fill */
-	of.modtype=strdup(GDM_Version);
+	of.modtype=StrDup(GDM_Version);
 	of.modtype[18]=mh->majorver+'0';
 	of.modtype[20]=mh->minorver/10+'0';
 	of.modtype[21]=mh->minorver%10+'0';
@@ -533,7 +533,7 @@ static BOOL GDM_Load(BOOL curious)
 	return 1;
 }
 
-static CHAR *GDM_LoadTitle(void)
+CHAR *GDM_LoadTitle(void)
 {
 	CHAR s[32];
 
