@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_imf.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Imago Orpheus (IMF) module loader
 
@@ -125,7 +125,7 @@ static	IMFHEADER *mh=NULL;
 
 /*========== Loader code */
 
-static BOOL IMF_Test(void)
+BOOL IMF_Test(void)
 {
 	UBYTE id[4];
 
@@ -135,20 +135,20 @@ static BOOL IMF_Test(void)
 	return 0;
 }
 
-static BOOL IMF_Init(void)
+BOOL IMF_Init(void)
 {
-	if(!(imfpat=(IMFNOTE*)_mm_malloc(32*256*sizeof(IMFNOTE)))) return 0;
-	if(!(mh=(IMFHEADER*)_mm_malloc(sizeof(IMFHEADER)))) return 0;
+	if(!(imfpat=(IMFNOTE*)MikMod_malloc(32*256*sizeof(IMFNOTE)))) return 0;
+	if(!(mh=(IMFHEADER*)MikMod_malloc(sizeof(IMFHEADER)))) return 0;
 
 	return 1;
 }
 
-static void IMF_Cleanup(void)
+void IMF_Cleanup(void)
 {
 	FreeLinear();
 
-	_mm_free(imfpat);
-	_mm_free(mh);
+	MikMod_free(imfpat);
+	MikMod_free(mh);
 }
 
 static BOOL IMF_ReadPattern(SLONG size,UWORD rows)
@@ -377,7 +377,7 @@ static UBYTE* IMF_ConvertTrack(IMFNOTE* tr,UWORD rows)
 	return UniDup();
 }
 
-static BOOL IMF_Load(BOOL curious)
+BOOL IMF_Load(BOOL curious)
 {
 #define IMF_SMPINCR 64
 	int t,u,track=0,oldnumsmp;
@@ -409,7 +409,7 @@ static BOOL IMF_Load(BOOL curious)
 
 	/* set module variables */
 	of.songname=DupStr(mh->songname,31,1);
-	of.modtype=strdup(IMF_Version);
+	of.modtype=StrDup(IMF_Version);
 	of.numpat=mh->patnum;
 	of.numins=mh->insnum;
 	of.reppos=0;
@@ -508,16 +508,16 @@ static BOOL IMF_Load(BOOL curious)
 		_mm_read_I_UWORDS(ih.panenv,IMFENVCNT,modreader);
 		_mm_read_I_UWORDS(ih.pitenv,IMFENVCNT,modreader);
 
-#if defined __STDC__ || defined _MSC_VER
+#if defined __STDC__ || defined _MSC_VER || defined MPW_C
 #define IMF_FinishLoadingEnvelope(name)					\
 		ih. name##pts=_mm_read_UBYTE(modreader);		\
 		ih. name##sus=_mm_read_UBYTE(modreader);		\
 		ih. name##beg=_mm_read_UBYTE(modreader);		\
 		ih. name##end=_mm_read_UBYTE(modreader);		\
 		ih. name##flg=_mm_read_UBYTE(modreader);		\
-		_mm_skip_BYTE(modreader);						\
-		_mm_skip_BYTE(modreader);						\
-		_mm_skip_BYTE(modreader)
+		_mm_read_UBYTE(modreader);						\
+		_mm_read_UBYTE(modreader);						\
+		_mm_read_UBYTE(modreader)
 #else
 #define IMF_FinishLoadingEnvelope(name)				\
 		ih. name/**/pts=_mm_read_UBYTE(modreader);	\
@@ -525,9 +525,9 @@ static BOOL IMF_Load(BOOL curious)
 		ih. name/**/beg=_mm_read_UBYTE(modreader);	\
 		ih. name/**/end=_mm_read_UBYTE(modreader);	\
 		ih. name/**/flg=_mm_read_UBYTE(modreader);	\
-		_mm_skip_BYTE(modreader);					\
-		_mm_skip_BYTE(modreader);					\
-		_mm_skip_BYTE(modreader)
+		_mm_read_UBYTE(modreader);					\
+		_mm_read_UBYTE(modreader);					\
+		_mm_read_UBYTE(modreader)
 #endif
 
 		IMF_FinishLoadingEnvelope(vol);
@@ -540,10 +540,10 @@ static BOOL IMF_Load(BOOL curious)
 		_mm_read_UBYTES(id,4,modreader);
 		/* Looks like Imago Orpheus forgets the signature for empty
 		   instruments following a multi-sample instrument... */
-		if(memcmp(id,"II10",4) &&
+		if(memcmp(id,"II10",4) && 
 		   (oldnumsmp && memcmp(id,"\x0\x0\x0\x0",4))) {
-			if(nextwav) free(nextwav);
-			if(wh) free(wh);
+			if(nextwav) MikMod_free(nextwav);
+			if(wh) MikMod_free(wh);
 			_mm_errno=MMERR_LOADING_SAMPLEINFO;
 			return 0;
 		}
@@ -551,8 +551,8 @@ static BOOL IMF_Load(BOOL curious)
 
 		if((ih.numsmp>16)||(ih.volpts>IMFENVCNT/2)||(ih.panpts>IMFENVCNT/2)||
 		   (ih.pitpts>IMFENVCNT/2)||(_mm_eof(modreader))) {
-			if(nextwav) free(nextwav);
-			if(wh) free(wh);
+			if(nextwav) MikMod_free(nextwav);
+			if(wh) MikMod_free(wh);
 			_mm_errno=MMERR_LOADING_SAMPLEINFO;
 			return 0;
 		}
@@ -561,7 +561,7 @@ static BOOL IMF_Load(BOOL curious)
 			d->samplenumber[u]=ih.what[u]>ih.numsmp?0xffff:ih.what[u]+of.numsmp;
 		d->volfade=ih.volfade;
 
-#if defined __STDC__ || defined _MSC_VER
+#if defined __STDC__ || defined _MSC_VER || defined MPW_C
 #define IMF_ProcessEnvelope(name) 									\
 		for (u = 0; u < (IMFENVCNT >> 1); u++) {					\
 			d-> name##env[u].pos = ih. name##env[u << 1];			\
@@ -612,13 +612,13 @@ static BOOL IMF_Load(BOOL curious)
 			/* allocate more room for sample information if necessary */
 			if(of.numsmp+u==wavcnt) {
 				wavcnt+=IMF_SMPINCR;
-				if(!(nextwav=realloc(nextwav,wavcnt*sizeof(ULONG)))) {
-					if(wh) free(wh);
+				if(!(nextwav=MikMod_realloc(nextwav,wavcnt*sizeof(ULONG)))) {
+					if(wh) MikMod_free(wh);
 					_mm_errno=MMERR_OUT_OF_MEMORY;
 					return 0;
 				}
-				if(!(wh=realloc(wh,wavcnt*sizeof(IMFWAVHEADER)))) {
-					free(nextwav);
+				if(!(wh=MikMod_realloc(wh,wavcnt*sizeof(IMFWAVHEADER)))) {
+					MikMod_free(nextwav);
 					_mm_errno=MMERR_OUT_OF_MEMORY;
 					return 0;
 				}
@@ -626,7 +626,7 @@ static BOOL IMF_Load(BOOL curious)
 			}
 
 			_mm_read_string(s->samplename,13,modreader);
-			_mm_skip_BYTE(modreader);_mm_skip_BYTE(modreader);_mm_skip_BYTE(modreader);
+			_mm_read_UBYTE(modreader);_mm_read_UBYTE(modreader);_mm_read_UBYTE(modreader);
 			s->length    =_mm_read_I_ULONG(modreader);
 			s->loopstart =_mm_read_I_ULONG(modreader);
 			s->loopend   =_mm_read_I_ULONG(modreader);
@@ -639,7 +639,7 @@ static BOOL IMF_Load(BOOL curious)
 			_mm_read_UBYTES(id,4,modreader);
 			if(((memcmp(id,"IS10",4))&&(memcmp(id,"IW10",4)))||
 			   (_mm_eof(modreader))) {
-				free(nextwav);free(wh);
+				MikMod_free(nextwav);MikMod_free(wh);
 				_mm_errno=MMERR_LOADING_SAMPLEINFO;
 				return 0;
 			}
@@ -653,19 +653,19 @@ static BOOL IMF_Load(BOOL curious)
 
 	/* sanity check */
 	if(!of.numsmp) {
-		if(nextwav) free(nextwav);
-		if(wh) free(wh);
+		if(nextwav) MikMod_free(nextwav);
+		if(wh) MikMod_free(wh);
 		_mm_errno=MMERR_LOADING_SAMPLEINFO;
 		return 0;
 	}
 
 	/* load samples */
 	if(!AllocSamples()) {
-		free(nextwav);free(wh);
+		MikMod_free(nextwav);MikMod_free(wh);
 		return 0;
 	}
 	if(!AllocLinear()) {
-		free(nextwav);free(wh);
+		MikMod_free(nextwav);MikMod_free(wh);
 		return 0;
 	}
 	q=of.samples;
@@ -708,11 +708,11 @@ static BOOL IMF_Load(BOOL curious)
 		}
 	}
 
-	free(wh);free(nextwav);
+	MikMod_free(wh);MikMod_free(nextwav);
 	return 1;
 }
 
-static CHAR *IMF_LoadTitle(void)
+CHAR *IMF_LoadTitle(void)
 {
 	CHAR s[31];
 
