@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_it.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Impulse tracker (IT) module loader
 
@@ -67,7 +67,7 @@ typedef struct ITHEADER {
 	UBYTE	initspeed;
 	UBYTE	inittempo;
 	UBYTE	pansep;		/* panning separation between channels */
-	UBYTE	zerobyte;
+	UBYTE	zerobyte;       
 	UWORD	msglength;
 	ULONG	msgoffset;
 	UBYTE	blank02[4];
@@ -107,19 +107,19 @@ typedef struct ITINSTHEADER {
 	CHAR	filename[12];	/* (char) Instrument filename */
 	UBYTE	zerobyte;		/* (byte) Instrument type (always 0) */
 	UBYTE	volflg;
-	UBYTE	volpts;
+	UBYTE	volpts;   
 	UBYTE	volbeg;			/* (byte) Volume loop start (node) */
 	UBYTE	volend;			/* (byte) Volume loop end (node) */
 	UBYTE	volsusbeg;		/* (byte) Volume sustain begin (node) */
 	UBYTE	volsusend;		/* (byte) Volume Sustain end (node) */
 	UBYTE	panflg;
-	UBYTE	panpts;
+	UBYTE	panpts;  
 	UBYTE	panbeg;			/* (byte) channel loop start (node) */
 	UBYTE	panend;			/* (byte) channel loop end (node) */
 	UBYTE	pansusbeg;		/* (byte) channel sustain begin (node) */
 	UBYTE	pansusend;		/* (byte) channel Sustain end (node) */
 	UBYTE	pitflg;
-	UBYTE	pitpts;
+	UBYTE	pitpts;   
 	UBYTE	pitbeg;			/* (byte) pitch loop start (node) */
 	UBYTE	pitend;			/* (byte) pitch loop end (node) */
 	UBYTE	pitsusbeg;		/* (byte) pitch sustain begin (node) */
@@ -149,7 +149,7 @@ typedef struct ITINSTHEADER {
 	UWORD	pantick[ITENVCNT];   /* tick value of panning nodes */
 	SBYTE	pitnode[ITENVCNT];   /* pitchenv - node points */
 	UWORD	pittick[ITENVCNT];   /* tick value of pitch nodes */
-} ITINSTHEADER;
+} ITINSTHEADER;                       
 
 /* unpacked note */
 
@@ -166,7 +166,7 @@ static UBYTE *mask=NULL;	/* arrays allocated to 64 elements and used for */
 static ITNOTE *last=NULL;	/* uncompressing IT's pattern information */
 static int numtrk=0;
 static unsigned int old_effect;		/* if set, use S3M old-effects stuffs */
-
+   
 static CHAR* IT_Version[]={
 	"ImpulseTracker  .  ",
 	"Compressed ImpulseTracker  .  ",
@@ -181,7 +181,7 @@ static UBYTE portatable[10]= {0,1,4,8,16,32,64,96,128,255};
 
 /*========== Loader code */
 
-static BOOL IT_Test(void)
+BOOL IT_Test(void)
 {
 	UBYTE id[4];
 
@@ -190,38 +190,38 @@ static BOOL IT_Test(void)
 	return 0;
 }
 
-static BOOL IT_Init(void)
+BOOL IT_Init(void)
 {
-	if(!(mh=(ITHEADER*)_mm_malloc(sizeof(ITHEADER)))) return 0;
-	if(!(poslookup=(UBYTE*)_mm_malloc(256*sizeof(UBYTE)))) return 0;
-	if(!(itpat=(ITNOTE*)_mm_malloc(200*64*sizeof(ITNOTE)))) return 0;
-	if(!(mask=(UBYTE*)_mm_malloc(64*sizeof(UBYTE)))) return 0;
-	if(!(last=(ITNOTE*)_mm_malloc(64*sizeof(ITNOTE)))) return 0;
+	if(!(mh=(ITHEADER*)MikMod_malloc(sizeof(ITHEADER)))) return 0;
+	if(!(poslookup=(UBYTE*)MikMod_malloc(256*sizeof(UBYTE)))) return 0;
+	if(!(itpat=(ITNOTE*)MikMod_malloc(200*64*sizeof(ITNOTE)))) return 0;
+	if(!(mask=(UBYTE*)MikMod_malloc(64*sizeof(UBYTE)))) return 0;
+	if(!(last=(ITNOTE*)MikMod_malloc(64*sizeof(ITNOTE)))) return 0;
 
 	return 1;
 }
 
-static void IT_Cleanup(void)
+void IT_Cleanup(void)
 {
 	FreeLinear();
 
-	_mm_free(mh);
-	_mm_free(poslookup);
-	_mm_free(itpat);
-	_mm_free(mask);
-	_mm_free(last);
-	_mm_free(paraptr);
-	_mm_free(origpositions);
+	MikMod_free(mh);
+	MikMod_free(poslookup);
+	MikMod_free(itpat);
+	MikMod_free(mask);
+	MikMod_free(last);
+	MikMod_free(paraptr);
+	MikMod_free(origpositions);
 }
 
 /* Because so many IT files have 64 channels as the set number used, but really
    only use far less (usually from 8 to 24 still), I had to make this function,
    which determines the number of channels that are actually USED by a pattern.
-
+ 
    NOTE: You must first seek to the file location of the pattern before calling
-   this procedure.
+         this procedure.
 
-   Returns 0 on error
+   Returns 1 on error
 */
 static BOOL IT_GetNumChannels(UWORD patrows)
 {
@@ -230,7 +230,7 @@ static BOOL IT_GetNumChannels(UWORD patrows)
 	do {
 		if((flag=_mm_read_UBYTE(modreader))==EOF) {
 			_mm_errno=MMERR_LOADING_PATTERN;
-			return 0;
+			return 1;
 		}
 		if(!flag)
 			row++;
@@ -238,14 +238,14 @@ static BOOL IT_GetNumChannels(UWORD patrows)
 			ch=(flag-1)&63;
 			remap[ch]=0;
 			if(flag & 128) mask[ch]=_mm_read_UBYTE(modreader);
-			if(mask[ch]&1)   _mm_skip_BYTE(modreader);
-			if(mask[ch]&2)   _mm_skip_BYTE(modreader);
-			if(mask[ch]&4)   _mm_skip_BYTE(modreader);
-			if(mask[ch]&8) { _mm_skip_BYTE(modreader);_mm_skip_BYTE(modreader); }
+			if(mask[ch]&1)   _mm_read_UBYTE(modreader);
+			if(mask[ch]&2)   _mm_read_UBYTE(modreader);
+			if(mask[ch]&4)   _mm_read_UBYTE(modreader);
+			if(mask[ch]&8) { _mm_read_UBYTE(modreader);_mm_read_UBYTE(modreader); }
 		}
 	} while(row<patrows);
 
-	return 1;
+	return 0;
 }
 
 static UBYTE* IT_ConvertTrack(ITNOTE* tr,UWORD numrows)
@@ -282,7 +282,7 @@ static UBYTE* IT_ConvertTrack(ITNOTE* tr,UWORD numrows)
 		/* process volume / panning column
 		   volume / panning effects do NOT all share the same memory address
 		   yet. */
-		if(volpan<=64)
+		if(volpan<=64) 
 			UniVolEffect(VOL_VOLUME,volpan);
 		else if(volpan==65) /* fine volume slide up (65-74) - A0 case */
 			UniVolEffect(VOL_VOLSLIDE,0);
@@ -342,13 +342,13 @@ static BOOL IT_ReadPattern(UWORD patrows)
 			if(ch!=-1) {
 				n=&itt[ch];
 				l=&last[ch];
-			} else
+			} else 
 				n=l=&dummy;
 
 			if(flag&128) mask[ch]=_mm_read_UBYTE(modreader);
 			if(mask[ch]&1)
 				/* convert IT note off to internal note off */
-				if((l->note=n->note=_mm_read_UBYTE(modreader))==255)
+				if((l->note=n->note=_mm_read_UBYTE(modreader))==255) 
 					l->note=n->note=253;
 			if(mask[ch]&2)
 				l->ins=n->ins=_mm_read_UBYTE(modreader);
@@ -379,38 +379,38 @@ static BOOL IT_ReadPattern(UWORD patrows)
 	return 1;
 }
 
-static void LoadMidiString(MREADER* r,CHAR* dest)
+static void LoadMidiString(MREADER* modreader,CHAR* dest)
 {
-	CHAR *curp,*lastp;
+	CHAR *cur,*last;
 
-	_mm_read_UBYTES(dest,32,r);
-	curp=lastp=dest;
+	_mm_read_UBYTES(dest,32,modreader);
+	cur=last=dest;
 	/* remove blanks and uppercase all */
-	while(*lastp) {
-		if(isalnum((int)*lastp)) *(curp++)=toupper((int)*lastp);
-		lastp++;
+	while(*last) {
+		if(isalnum((int)*last)) *(cur++)=toupper((int)*last);
+		last++;
 	}
-	*curp=0;
+	*cur=0;
 }
 
 /* Load embedded midi information for resonant filters */
-static void IT_LoadMidiConfiguration(MREADER* r)
+static void IT_LoadMidiConfiguration(MREADER* modreader)
 {
 	int i;
 
 	memset(filtermacros,0,sizeof(filtermacros));
 	memset(filtersettings,0,sizeof(filtersettings));
 
-	if (r) { /* information is embedded in file */
+	if (modreader) { /* information is embedded in file */
 		UWORD dat;
 		CHAR midiline[33];
 
-		dat=_mm_read_I_UWORD(r);
-		_mm_fseek(r,8*dat+0x120,SEEK_CUR);
+		dat=_mm_read_I_UWORD(modreader);
+		_mm_fseek(modreader,8*dat+0x120,SEEK_CUR);
 
 		/* read midi macros */
 		for(i=0;i<UF_MAXMACRO;i++) {
-			LoadMidiString(r,midiline);
+			LoadMidiString(modreader,midiline);
 			if((!strncmp(midiline,"F0F00",5))&&
 			   ((midiline[5]=='0')||(midiline[5]=='1')))
 					filtermacros[i]=(midiline[5]-'0')|0x80;
@@ -418,7 +418,7 @@ static void IT_LoadMidiConfiguration(MREADER* r)
 
 		/* read standalone filters */
 		for(i=0x80;i<0x100;i++) {
-			LoadMidiString(r,midiline);
+			LoadMidiString(modreader,midiline);
 			if((!strncmp(midiline,"F0F00",5))&&
 			   ((midiline[5]=='0')||(midiline[5]=='1'))) {
 				filtersettings[i].filter=(midiline[5]-'0')|0x80;
@@ -441,12 +441,12 @@ static void IT_LoadMidiConfiguration(MREADER* r)
 	}
 }
 
-static BOOL IT_Load(BOOL curious)
+BOOL IT_Load(BOOL curious)
 {
 	int t,u,lp;
 	INSTRUMENT *d;
 	SAMPLE *q;
-	/*BOOL compressed=0;*/
+	BOOL compressed=0;
 
 	numtrk=0;
 	filters=0;
@@ -506,11 +506,11 @@ static BOOL IT_Load(BOOL curious)
 	/* 2.16 : IT 2.14p3 with resonant filters */
 	/* 2.15 : IT 2.14p3 (improved compression) */
 	if((mh->cwt<=0x219)&&(mh->cwt>=0x217))
-		of.modtype=strdup(IT_Version[mh->cmwt<0x214?4:5]);
+		of.modtype=StrDup(IT_Version[mh->cmwt<0x214?4:5]);
 	else if (mh->cwt>=0x215)
-		of.modtype=strdup(IT_Version[mh->cmwt<0x214?2:3]);
+		of.modtype=StrDup(IT_Version[mh->cmwt<0x214?2:3]);
 	else {
-		of.modtype = strdup(IT_Version[mh->cmwt<0x214?0:1]);
+		of.modtype     = StrDup(IT_Version[mh->cmwt<0x214?0:1]);
 		of.modtype[mh->cmwt<0x214?15:26] = (mh->cwt>>8)+'0';
 		of.modtype[mh->cmwt<0x214?17:28] = ((mh->cwt>>4)&0xf)+'0';
 		of.modtype[mh->cmwt<0x214?18:29] = ((mh->cwt)&0xf)+'0';
@@ -550,7 +550,7 @@ static BOOL IT_Load(BOOL curious)
 
 	/* read the order data */
 	if(!AllocPositions(mh->ordnum)) return 0;
-	if(!(origpositions=_mm_calloc(mh->ordnum,sizeof(UWORD)))) return 0;
+	if(!(origpositions=MikMod_calloc(mh->ordnum,sizeof(UWORD)))) return 0;
 
 	for(t=0;t<mh->ordnum;t++) {
 		origpositions[t]=_mm_read_UBYTE(modreader);
@@ -566,7 +566,7 @@ static BOOL IT_Load(BOOL curious)
 	poslookupcnt=mh->ordnum;
 	S3MIT_CreateOrders(curious);
 
-	if(!(paraptr=(ULONG*)_mm_malloc((mh->insnum+mh->smpnum+of.numpat)*
+	if(!(paraptr=(ULONG*)MikMod_malloc((mh->insnum+mh->smpnum+of.numpat)*
 	                               sizeof(ULONG)))) return 0;
 
 	/* read the instrument, sample, and pattern parapointers */
@@ -672,14 +672,14 @@ static BOOL IT_Load(BOOL curious)
 		if(s.flag&2) q->flags|=SF_16BITS;
 		if((s.flag&8)&&(mh->cwt>=0x214)) {
 			q->flags|=SF_ITPACKED;
-			/*compressed=1;*/
+			compressed=1;
 		}
 		if(s.flag&16) q->flags|=SF_LOOP;
 		if(s.flag&64) q->flags|=SF_BIDI;
 
 		if(mh->cwt>=0x200) {
 			if(s.convert&1) q->flags|=SF_SIGNED;
-			if(s.convert&4) q->flags|=SF_DELTA;
+			if(s.convert&4) q->flags|=SF_DELTA;   
 		}
 		q++;
 	}
@@ -726,7 +726,7 @@ static BOOL IT_Load(BOOL curious)
 
 			ih.trkvers   = _mm_read_I_UWORD(modreader);
 			ih.numsmp    = _mm_read_UBYTE(modreader);
-			_mm_skip_BYTE(modreader);
+			_mm_read_UBYTE(modreader);
 			_mm_read_string(ih.name,26,modreader);
 			_mm_read_UBYTES(ih.blank01,6,modreader);
 			_mm_read_I_UWORDS(ih.samptable,ITNOTECNT,modreader);
@@ -736,10 +736,10 @@ static BOOL IT_Load(BOOL curious)
 				for(lp=0;lp<ITENVCNT;lp++) {
 					ih.oldvoltick[lp] = _mm_read_UBYTE(modreader);
 					ih.volnode[lp]    = _mm_read_UBYTE(modreader);
-				}
+				} 
 			} else {
 				/* load IT 2xx volume, pan and pitch envelopes */
-#if defined __STDC__ || defined _MSC_VER
+#if defined __STDC__ || defined _MSC_VER || defined MPW_C
 #define IT_LoadEnvelope(name,type) 										\
 				ih. name##flg   =_mm_read_UBYTE(modreader);				\
 				ih. name##pts   =_mm_read_UBYTE(modreader);				\
@@ -753,7 +753,7 @@ static BOOL IT_Load(BOOL curious)
 					ih. name##node[lp]=_mm_read_##type (modreader);		\
 					ih. name##tick[lp]=_mm_read_I_UWORD(modreader);		\
 				}														\
-				_mm_skip_BYTE(modreader)
+				_mm_read_UBYTE(modreader)
 #else
 #define IT_LoadEnvelope(name,type) 										\
 				ih. name/**/flg   =_mm_read_UBYTE(modreader);			\
@@ -768,7 +768,7 @@ static BOOL IT_Load(BOOL curious)
 					ih. name/**/node[lp]=_mm_read_/**/type (modreader);	\
 					ih. name/**/tick[lp]=_mm_read_I_UWORD(modreader);	\
 				}														\
-				_mm_skip_BYTE(modreader)
+				_mm_read_UBYTE(modreader)
 #endif
 
 				IT_LoadEnvelope(vol,UBYTE);
@@ -776,7 +776,7 @@ static BOOL IT_Load(BOOL curious)
 				IT_LoadEnvelope(pit,SBYTE);
 #undef IT_LoadEnvelope
 			}
-
+ 
 			if(_mm_eof(modreader)) {
 				_mm_errno = MMERR_LOADING_SAMPLEINFO;
 				return 0;
@@ -795,10 +795,10 @@ static BOOL IT_Load(BOOL curious)
 
 				if(ih.volflg&1) d->volflg|=EF_ON;
 				if(ih.volflg&2) d->volflg|=EF_LOOP;
-				if(ih.volflg&4) d->volflg|=EF_SUSTAIN;
+				if(ih.volflg&4) d->volflg|=EF_SUSTAIN;      
 
 				/* XM conversion of IT envelope Array */
-				d->volbeg    = ih.volbeg;
+				d->volbeg    = ih.volbeg;   
 				d->volend    = ih.volend;
 				d->volsusbeg = ih.volsusbeg;
 				d->volsusend = ih.volsusend;
@@ -811,7 +811,7 @@ static BOOL IT_Load(BOOL curious)
 							d->volpts++;
 						} else
 							break;
-				}
+				}  
 			} else {
 				d->panning=((ih.chanpan&127)==64)?255:(ih.chanpan&127)<<2;
 				if(!(ih.chanpan&128)) d->flags|=IF_OWNPAN;
@@ -831,7 +831,7 @@ static BOOL IT_Load(BOOL curious)
 					d->rpanvar = ih.rpanvar;
 				}
 
-#if defined __STDC__ || defined _MSC_VER
+#if defined __STDC__ || defined _MSC_VER || defined MPW_C
 #define IT_ProcessEnvelope(name) 										\
 				if(ih. name##flg&1) d-> name##flg|=EF_ON;				\
 				if(ih. name##flg&2) d-> name##flg|=EF_LOOP;				\
@@ -866,7 +866,7 @@ static BOOL IT_Load(BOOL curious)
 #endif
 
 				IT_ProcessEnvelope(vol);
-
+			
 				for(u=0;u<ih.volpts;u++)
 					d->volenv[u].val=(ih.volnode[u]<<2);
 
@@ -887,7 +887,7 @@ static BOOL IT_Load(BOOL curious)
 #ifdef MIKMOD_DEBUG
 					{
 						static int warn=0;
-
+						
 						if(!warn)
 							fprintf(stderr, "\rFilter envelopes not supported yet\n");
 						warn=1;
@@ -908,7 +908,7 @@ static BOOL IT_Load(BOOL curious)
 				}
 			}
 
-			d++;
+			d++;                  
 		}
 	} else if(of.flags & UF_LINEAR) {
 		if(!AllocInstruments()) return 0;
@@ -946,12 +946,12 @@ static BOOL IT_Load(BOOL curious)
 				return 0;
 			}
 			_mm_read_I_ULONG(modreader);
-			if(!IT_GetNumChannels(packlen)) return 0;
+			if(IT_GetNumChannels(packlen)) return 0;
 		}
 	}
 
 	/* give each of them a different number */
-	for(t=0;t<UF_MAXCHAN;t++)
+	for(t=0;t<UF_MAXCHAN;t++) 
 		if(!remap[t])
 			remap[t]=of.numchn++;
 
@@ -978,7 +978,6 @@ static BOOL IT_Load(BOOL curious)
 		} else {
 			_mm_fseek(modreader,((long)paraptr[mh->insnum+mh->smpnum+t]),SEEK_SET);
 			packlen=_mm_read_I_UWORD(modreader);
-			(void)packlen; /* unused */
 			of.pattrows[t]=_mm_read_I_UWORD(modreader);
 			_mm_read_I_ULONG(modreader);
 			if(!IT_ReadPattern(of.pattrows[t])) return 0;
@@ -988,13 +987,13 @@ static BOOL IT_Load(BOOL curious)
 	return 1;
 }
 
-static CHAR *IT_LoadTitle(void)
+CHAR *IT_LoadTitle(void)
 {
 	CHAR s[26];
 
 	_mm_fseek(modreader,4,SEEK_SET);
 	if(!_mm_read_UBYTES(s,26,modreader)) return NULL;
-
+   
 	return(DupStr(s,26,0));
 }
 
