@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_okt.c,v 1.1.1.1 2004/06/01 12:16:17 raph Exp $
+  $Id$
 
   Oktalyzer (OKT) module loader
 
@@ -74,7 +74,7 @@ static OKTNOTE *okttrk = NULL;
 
 /*========== Loader code */
 
-static BOOL OKT_Test(void)
+BOOL OKT_Test(void)
 {
 	CHAR id[8];
 
@@ -203,7 +203,7 @@ static BOOL OKT_doSAMP(int len)
 		s.len = _mm_read_M_ULONG(modreader);
 		s.loopbeg = _mm_read_M_UWORD(modreader) * 2;
 		s.looplen = _mm_read_M_UWORD(modreader) * 2;
-		_mm_skip_BYTE(modreader);
+		_mm_read_UBYTE(modreader);
 		s.volume = _mm_read_UBYTE(modreader);
 		_mm_read_M_UWORD(modreader);
 
@@ -292,8 +292,8 @@ static BOOL OKT_doPBOD(int patnum)
 	/* Read pattern */
 	of.pattrows[patnum] = rows = _mm_read_M_UWORD(modreader);
 
-	if (!(okttrk = (OKTNOTE *) _mm_calloc(rows, sizeof(OKTNOTE))) ||
-	    !(patbuf = (char *)_mm_calloc(rows * of.numchn, sizeof(OKTNOTE))))
+	if (!(okttrk = (OKTNOTE *) MikMod_calloc(rows, sizeof(OKTNOTE))) ||
+	    !(patbuf = (char *)MikMod_calloc(rows * of.numchn, sizeof(OKTNOTE))))
 		return 0;
 	_mm_read_UBYTES(patbuf, rows * of.numchn * sizeof(OKTNOTE), modreader);
 	if (_mm_eof(modreader)) {
@@ -312,8 +312,8 @@ static BOOL OKT_doPBOD(int patnum)
 		if (!(of.tracks[patnum * of.numchn + i] = OKT_ConvertTrack(rows)))
 			return 0;
 	}
-	_mm_free(patbuf);
-	_mm_free(okttrk);
+	MikMod_free(patbuf);
+	MikMod_free(okttrk);
 	return 1;
 }
 
@@ -322,7 +322,7 @@ static void OKT_doSBOD(int insnum)
 	of.samples[insnum].seekpos = _mm_ftell(modreader);
 }
 
-static BOOL OKT_Load(BOOL curious)
+BOOL OKT_Load(BOOL curious)
 {
 	UBYTE id[4];
 	ULONG len;
@@ -333,24 +333,24 @@ static BOOL OKT_Load(BOOL curious)
 
 	/* skip OKTALYZER header */
 	_mm_fseek(modreader, 8, SEEK_SET);
-	of.songname = strdup("");
+	of.songname = StrDup("");
 
-	of.modtype = strdup("Amiga Oktalyzer");
+	of.modtype = StrDup("Amiga Oktalyzer");
 	of.numpos = of.reppos = 0;
-
+	
 	/* default values */
 	of.initspeed = 6;
 	of.inittempo = 125;
-
+	
 	while (1) {
 		/* read block header */
 		_mm_read_UBYTES(id, 4, modreader);
 		len = _mm_read_M_ULONG(modreader);
-
+		
 		if (_mm_eof(modreader))
 			break;
 		fp = _mm_ftell(modreader);
-
+		
 		if (!memcmp(id, "CMOD", 4)) {
 			if (!seen_cmod) {
 				OKT_doCMOD();
@@ -439,9 +439,9 @@ static BOOL OKT_Load(BOOL curious)
 	return 1;
 }
 
-static CHAR *OKT_LoadTitle(void)
+CHAR *OKT_LoadTitle(void)
 {
-	return strdup("");
+	return StrDup("");
 }
 
 /*========== Loader information */
