@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: load_xm.c,v 1.1.1.2 2007/12/15 15:24:17 denis111 Exp $
+  $Id$
 
   Fasttracker (XM) module loader
 
@@ -59,7 +59,7 @@ typedef struct XMHEADER {
 	UWORD numchn;          /* Number of channels (2,4,6,8,10,...,32) */
 	UWORD numpat;          /* Number of patterns (max 256) */
 	UWORD numins;          /* Number of instruments (max 128) */
-	UWORD flags;
+	UWORD flags;       
 	UWORD tempo;           /* Default tempo */
 	UWORD bpm;             /* Default BPM */
 	UBYTE orders[256];     /* Pattern order table  */
@@ -129,14 +129,14 @@ typedef struct XMNOTE {
 static	XMNOTE *xmpat=NULL;
 static	XMHEADER *mh=NULL;
 
-/* increment unit for sample array reallocation */
+/* increment unit for sample array MikMod_reallocation */
 #define XM_SMPINCR 64
 static	ULONG *nextwav=NULL;
 static	XMWAVHEADER *wh=NULL,*s=NULL;
 
 /*========== Loader code */
 
-static BOOL XM_Test(void)
+BOOL XM_Test(void)
 {
 	UBYTE id[38];
 
@@ -146,15 +146,15 @@ static BOOL XM_Test(void)
 	return 0;
 }
 
-static BOOL XM_Init(void)
+BOOL XM_Init(void)
 {
-	if(!(mh=(XMHEADER *)_mm_malloc(sizeof(XMHEADER)))) return 0;
+	if(!(mh=(XMHEADER *)MikMod_malloc(sizeof(XMHEADER)))) return 0;
 	return 1;
 }
 
-static void XM_Cleanup(void)
+void XM_Cleanup(void)
 {
-	_mm_free(mh);
+	MikMod_free(mh);
 }
 
 static int XM_ReadNote(XMNOTE* n)
@@ -355,18 +355,18 @@ static BOOL LoadPatterns(BOOL dummypat)
 		of.pattrows[t]=ph.numrows;
 
 		if(ph.numrows) {
-			if(!(xmpat=(XMNOTE*)_mm_calloc(ph.numrows*of.numchn,sizeof(XMNOTE))))
+			if(!(xmpat=(XMNOTE*)MikMod_calloc(ph.numrows*of.numchn,sizeof(XMNOTE))))
 				return 0;
 
 			/* when packsize is 0, don't try to load a pattern.. it's empty. */
-			if(ph.packsize)
-				for(u=0;u<ph.numrows;u++)
+			if(ph.packsize) 
+				for(u=0;u<ph.numrows;u++) 
 					for(v=0;v<of.numchn;v++) {
 						if(!ph.packsize) break;
 
 						ph.packsize-=XM_ReadNote(&xmpat[(v*ph.numrows)+u]);
 						if(ph.packsize<0) {
-							free(xmpat);xmpat=NULL;
+							MikMod_free(xmpat);xmpat=NULL;
 							_mm_errno=MMERR_LOADING_PATTERN;
 							return 0;
 						}
@@ -377,7 +377,7 @@ static BOOL LoadPatterns(BOOL dummypat)
 			}
 
 			if(_mm_eof(modreader)) {
-				free(xmpat);xmpat=NULL;
+				MikMod_free(xmpat);xmpat=NULL;
 				_mm_errno=MMERR_LOADING_PATTERN;
 				return 0;
 			}
@@ -385,7 +385,7 @@ static BOOL LoadPatterns(BOOL dummypat)
 			for(v=0;v<of.numchn;v++)
 				of.tracks[numtrk++]=XM_Convert(&xmpat[v*ph.numrows],ph.numrows);
 
-			free(xmpat);xmpat=NULL;
+			MikMod_free(xmpat);xmpat=NULL;
 		} else {
 			for(v=0;v<of.numchn;v++)
 				of.tracks[numtrk++]=XM_Convert(NULL,ph.numrows);
@@ -394,10 +394,10 @@ static BOOL LoadPatterns(BOOL dummypat)
 
 	if(dummypat) {
 		of.pattrows[t]=64;
-		if(!(xmpat=(XMNOTE*)_mm_calloc(64*of.numchn,sizeof(XMNOTE)))) return 0;
+		if(!(xmpat=(XMNOTE*)MikMod_calloc(64*of.numchn,sizeof(XMNOTE)))) return 0;
 		for(v=0;v<of.numchn;v++)
 			of.tracks[numtrk++]=XM_Convert(&xmpat[v*64],64);
-		free(xmpat);xmpat=NULL;
+		MikMod_free(xmpat);xmpat=NULL;
 	}
 
 	return 1;
@@ -500,11 +500,7 @@ static BOOL LoadInstruments(void)
 
 				/* read the remainder of the header
 				   (2 bytes for 1.03, 22 for 1.04) */
-				if (headend>=_mm_ftell(modreader)) {
-					for(u=headend-_mm_ftell(modreader);u;u--) {
-						_mm_skip_BYTE(modreader);
-					}
-				}
+				if (headend>=_mm_ftell(modreader)) for(u=headend-_mm_ftell(modreader);u;u--) _mm_read_UBYTE(modreader);
 
 				/* we can't trust the envelope point count here, as some
 				   modules have incorrect values (K_OSPACE.XM reports 32 volume
@@ -513,8 +509,8 @@ static BOOL LoadInstruments(void)
 				if(pth.panpts>XMENVCNT/2) pth.panpts=XMENVCNT/2;
 
 				if((_mm_eof(modreader))||(pth.volpts>XMENVCNT/2)||(pth.panpts>XMENVCNT/2)) {
-					if(nextwav) { free(nextwav);nextwav=NULL; }
-					if(wh) { free(wh);wh=NULL; }
+					if(nextwav) { MikMod_free(nextwav);nextwav=NULL; }
+					if(wh) { MikMod_free(wh);wh=NULL; }
 					_mm_errno = MMERR_LOADING_SAMPLEINFO;
 					return 0;
 				}
@@ -523,7 +519,7 @@ static BOOL LoadInstruments(void)
 					d->samplenumber[u]=pth.what[u]+of.numsmp;
 				d->volfade = pth.volfade;
 
-#if defined __STDC__ || defined _MSC_VER
+#if defined __STDC__ || defined _MSC_VER || defined MPW_C
 #define XM_ProcessEnvelope(name) 										\
 				for (u = 0; u < (XMENVCNT >> 1); u++) {					\
 					d-> name##env[u].pos = pth. name##env[u << 1];		\
@@ -564,7 +560,7 @@ static BOOL LoadInstruments(void)
 																			\
 				if ((d-> name/**/flg&EF_ON)&&(d-> name/**/pts<2))			\
 					d-> name/**/flg&=~EF_ON
-#endif
+#endif			
 
 				XM_ProcessEnvelope(vol);
 				XM_ProcessEnvelope(pan);
@@ -584,13 +580,13 @@ static BOOL LoadInstruments(void)
 					/* Allocate more room for sample information if necessary */
 					if(of.numsmp+u==wavcnt) {
 						wavcnt+=XM_SMPINCR;
-						if(!(nextwav=realloc(nextwav,wavcnt*sizeof(ULONG)))){
-							if(wh) { free(wh);wh=NULL; }
+						if(!(nextwav=MikMod_realloc(nextwav,wavcnt*sizeof(ULONG)))){
+							if(wh) { MikMod_free(wh);wh=NULL; }
 							_mm_errno = MMERR_OUT_OF_MEMORY;
 							return 0;
 						}
-						if(!(wh=realloc(wh,wavcnt*sizeof(XMWAVHEADER)))) {
-							free(nextwav);nextwav=NULL;
+						if(!(wh=MikMod_realloc(wh,wavcnt*sizeof(XMWAVHEADER)))) {
+							MikMod_free(nextwav);nextwav=NULL;
 							_mm_errno = MMERR_OUT_OF_MEMORY;
 							return 0;
 						}
@@ -615,9 +611,8 @@ static BOOL LoadInstruments(void)
 					nextwav[of.numsmp+u]=next;
 					next+=s->length;
 
-					/* last instrument is at the end of file in version 0x0104 */
-					if(_mm_eof(modreader) && (mh->version<0x0104 || t<of.numins-1)) {
-						free(nextwav);free(wh);
+					if(_mm_eof(modreader)) {
+						MikMod_free(nextwav);MikMod_free(wh);
 						nextwav=NULL;wh=NULL;
 						_mm_errno = MMERR_LOADING_SAMPLEINFO;
 						return 0;
@@ -639,13 +634,11 @@ static BOOL LoadInstruments(void)
 					break;
 				}
 				_mm_fseek(modreader,ck,SEEK_SET);
-				for(u=headend-_mm_ftell(modreader);u;u--) {
-					_mm_skip_BYTE(modreader);
-				}
+				for(u=headend-_mm_ftell(modreader);u;u--) _mm_read_UBYTE(modreader);
 
 				/* last instrument is at the end of file in version 0x0104 */
 				if(_mm_eof(modreader) && (mh->version<0x0104 || t<of.numins-1)) {
-					free(nextwav);free(wh);
+					MikMod_free(nextwav);MikMod_free(wh);
 					nextwav=NULL;wh=NULL;
 					_mm_errno = MMERR_LOADING_SAMPLEINFO;
 					return 0;
@@ -656,8 +649,8 @@ static BOOL LoadInstruments(void)
 
 	/* sanity check */
 	if(!of.numsmp) {
-		if(nextwav) { free(nextwav);nextwav=NULL; }
-		if(wh) { free(wh);wh=NULL; }
+		if(nextwav) { MikMod_free(nextwav);nextwav=NULL; }
+		if(wh) { MikMod_free(wh);wh=NULL; }
 		_mm_errno = MMERR_LOADING_SAMPLEINFO;
 		return 0;
 	}
@@ -665,7 +658,7 @@ static BOOL LoadInstruments(void)
 	return 1;
 }
 
-static BOOL XM_Load(BOOL curious)
+BOOL XM_Load(BOOL curious)
 {
 	INSTRUMENT *d;
 	SAMPLE *q;
@@ -691,21 +684,19 @@ static BOOL XM_Load(BOOL curious)
 	mh->flags       =_mm_read_I_UWORD(modreader);
 	mh->tempo       =_mm_read_I_UWORD(modreader);
 	mh->bpm         =_mm_read_I_UWORD(modreader);
-	if(!mh->bpm || mh->songlength > 256) {
+	if(!mh->bpm) {
 		_mm_errno=MMERR_NOT_A_MODULE;
 		return 0;
 	}
-/*	_mm_read_UBYTES(mh->orders,256,modreader);*/
-/*	_mm_read_UBYTES(mh->orders,mh->headersize-20,modreader);*/
-	_mm_read_UBYTES(mh->orders,mh->songlength,modreader);
-	if(_mm_fseek(modreader, mh->headersize+60, SEEK_SET) ||
-	   _mm_eof(modreader)) {
+	_mm_read_UBYTES(mh->orders,mh->headersize-20,modreader);
+
+	if(_mm_eof(modreader)) {
 		_mm_errno = MMERR_LOADING_HEADER;
 		return 0;
 	}
 
 	/* set module variables */
-	of.initspeed = mh->tempo;
+	of.initspeed = mh->tempo;         
 	of.inittempo = mh->bpm;
 	strncpy(tracker,mh->trackername,20);tracker[20]=0;
 	for(t=20;(tracker[t]<=' ')&&(t>=0);t--) tracker[t]=0;
@@ -721,7 +712,7 @@ static BOOL XM_Load(BOOL curious)
 	sprintf(modtype,"%s (XM format %d.%02d)",
 	                tracker,mh->version>>8,mh->version&0xff);
 #endif
-	of.modtype   = strdup(modtype);
+	of.modtype   = StrDup(modtype);
 	of.numchn    = mh->numchn;
 	of.numpat    = mh->numpat;
 	of.numtrk    = (UWORD)of.numpat*of.numchn;   /* get number of channels */
@@ -764,7 +755,7 @@ static BOOL XM_Load(BOOL curious)
 	}
 
 	if(!AllocSamples()) {
-		free(nextwav);free(wh);
+		MikMod_free(nextwav);MikMod_free(wh);
 		nextwav=NULL;wh=NULL;
 		return 0;
 	}
@@ -808,19 +799,19 @@ static BOOL XM_Load(BOOL curious)
 			}
 		}
 
-	free(wh);free(nextwav);
+	MikMod_free(wh);MikMod_free(nextwav);
 	wh=NULL;nextwav=NULL;
 	return 1;
 }
 
-static CHAR *XM_LoadTitle(void)
+CHAR *XM_LoadTitle(void)
 {
-	CHAR str[21];
+	CHAR s[21];
 
 	_mm_fseek(modreader,17,SEEK_SET);
-	if(!_mm_read_UBYTES(str,21,modreader)) return NULL;
+	if(!_mm_read_UBYTES(s,21,modreader)) return NULL;
 
-	return(DupStr(str,21,1));
+	return(DupStr(s,21,1));
 }
 
 /*========== Loader information */
