@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mdreg.c,v 1.1.1.1 2004/06/01 12:16:18 raph Exp $
+  $Id$
 
   Routine for registering all drivers in libmikmod for the current platform.
 
@@ -32,7 +32,7 @@
 
 #include "mikmod_internals.h"
 
-static void _mm_registeralldrivers(void)
+void _mm_registeralldrivers(void)
 {
 	/* Register network drivers */
 #ifdef DRV_AF
@@ -41,13 +41,16 @@ static void _mm_registeralldrivers(void)
 #ifdef DRV_ESD
 	_mm_registerdriver(&drv_esd);
 #endif
+#ifdef DRV_NAS
+	_mm_registerdriver(&drv_nas);
+#endif
 
 	/* Register hardware drivers - hardware mixing */
 #ifdef DRV_ULTRA
 	_mm_registerdriver(&drv_ultra);
 #endif
 
-	/* Register OS-specific hardware drivers - software mixing */
+	/* Register hardware drivers - software mixing */
 #ifdef DRV_AIX
 	_mm_registerdriver(&drv_aix);
 #endif
@@ -81,24 +84,37 @@ static void _mm_registeralldrivers(void)
 #ifdef DRV_MAC
 	_mm_registerdriver(&drv_mac);
 #endif
-
+#ifdef DRV_OSX
+	_mm_registerdriver(&drv_osx);
+#endif
+#ifdef DRV_GP32
+	_mm_registerdriver(&drv_gp32);
+#endif
+	
+	/* dos drivers */
+#ifdef DRV_WSS
+	/* wss first, since some cards emulate sb */
+	_mm_registerdriver(&drv_wss);
+#endif
+#ifdef DRV_SB
+	_mm_registerdriver(&drv_sb);
+#endif
+	
 	/* Register disk writers */
-#ifdef DRV_WAV
-	_mm_registerdriver(&drv_wav);
-#endif
-#ifdef DRV_RAW
 	_mm_registerdriver(&drv_raw);
+	_mm_registerdriver(&drv_wav);
+#ifdef DRV_AIFF
+	_mm_registerdriver(&drv_aiff);
 #endif
-
+	
 	/* Register other drivers */
 #ifdef DRV_PIPE
 	_mm_registerdriver(&drv_pipe);
 #endif
-#if defined(DRV_STDOUT) && !defined(macintosh)
+#ifndef macintosh
 	_mm_registerdriver(&drv_stdout);
 #endif
 
-	/* Register 'nosound' driver */
 	_mm_registerdriver(&drv_nos);
 }
 
