@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mlreg.c,v 1.1.1.1 2004/06/01 12:16:18 raph Exp $
+  $Id$
 
   Routine for registering all loaders in libmikmod for the current platform.
 
@@ -32,13 +32,15 @@
 
 #include "mikmod_internals.h"
 
-static void MikMod_RegisterAllLoaders_internal(void)
+void MikMod_RegisterAllLoaders_internal(void)
 {
 	_mm_registerloader(&load_669);
 	_mm_registerloader(&load_amf);
+	_mm_registerloader(&load_asy);
 	_mm_registerloader(&load_dsm);
 	_mm_registerloader(&load_far);
 	_mm_registerloader(&load_gdm);
+	_mm_registerloader(&load_gt2);
 	_mm_registerloader(&load_it);
 	_mm_registerloader(&load_imf);
 	_mm_registerloader(&load_mod);
