@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mlutil.c,v 1.1.1.2 2007/12/15 15:24:17 denis111 Exp $
+  $Id$
 
   Utility functions for the module loader
 
@@ -43,7 +43,7 @@ extern int fprintf(FILE *, const char *, ...);
 
 /*========== Shared tracker identifiers */
 
-const CHAR *STM_Signatures[STM_NTRACKERS] = {
+CHAR *STM_Signatures[STM_NTRACKERS] = {
 	"!Scream!",
 	"BMOD2STM",
 	"WUZAMOD!"
@@ -71,13 +71,13 @@ FILTER filtersettings[UF_MAXFILTER]; /* computed filter settings */
 /*========== Linear periods stuff */
 
 int*   noteindex=NULL;      /* remap value for linear period modules */
-static unsigned noteindexcount=0;
+static int noteindexcount=0;
 
 int *AllocLinear(void)
 {
 	if(of.numsmp>noteindexcount) {
 		noteindexcount=of.numsmp;
-		noteindex=realloc(noteindex,noteindexcount*sizeof(int));
+		noteindex=MikMod_realloc(noteindex,noteindexcount*sizeof(int));
 	}
 	return noteindex;
 }
@@ -85,7 +85,7 @@ int *AllocLinear(void)
 void FreeLinear(void)
 {
 	if(noteindex) {
-		free(noteindex);
+		MikMod_free(noteindex);
 		noteindex=NULL;
 	}
 	noteindexcount=0;
@@ -93,7 +93,7 @@ void FreeLinear(void)
 
 int speed_to_finetune(ULONG speed,int sample)
 {
-    int ctmp=0,tmp,note=1,ft=0;
+    int ctmp=0,tmp,note=1,finetune=0;
 
     speed>>=1;
     while((tmp=getfrequency(of.flags,getlinearperiod(note<<1,0)))<speed) {
@@ -104,16 +104,16 @@ int speed_to_finetune(ULONG speed,int sample)
     if(tmp!=speed) {
         if((tmp-speed)<(speed-ctmp))
             while(tmp>speed)
-                tmp=getfrequency(of.flags,getlinearperiod(note<<1,--ft));
+                tmp=getfrequency(of.flags,getlinearperiod(note<<1,--finetune));
         else {
             note--;
             while(ctmp<speed)
-                ctmp=getfrequency(of.flags,getlinearperiod(note<<1,++ft));
+                ctmp=getfrequency(of.flags,getlinearperiod(note<<1,++finetune));
         }
     }
 
     noteindex[sample]=note-4*OCTAVE;
-    return ft;
+    return finetune;
 }
 
 /*========== Order stuff */
@@ -143,8 +143,10 @@ void S3MIT_CreateOrders(BOOL curious)
 /* handles S3M and IT effects */
 void S3MIT_ProcessCmd(UBYTE cmd,UBYTE inf,unsigned int flags)
 {
-	UBYTE lo = inf&0xF;
-	/*UBYTE hi = inf>>4;*/
+	UBYTE hi,lo;
+
+	lo=inf&0xf;
+	hi=inf>>4;
 
 	/* process S3M / IT specific command structure */
 
@@ -194,7 +196,7 @@ void S3MIT_ProcessCmd(UBYTE cmd,UBYTE inf,unsigned int flags)
 			case 9: /* Ixy tremor, ontime x, offtime y */
 				if (flags & S3MIT_OLDSTYLE)
 					UniEffect(UNI_S3MEFFECTI,inf);
-				else
+				else                     
 					UniEffect(UNI_ITEFFECTI,inf);
 				break;
 			case 0xa: /* Jxy arpeggio */
@@ -202,7 +204,7 @@ void S3MIT_ProcessCmd(UBYTE cmd,UBYTE inf,unsigned int flags)
 				break;
 			case 0xb: /* Kxy Dual command H00 & Dxy */
 				if (flags & S3MIT_OLDSTYLE)
-					UniPTEffect(0x4,0);
+					UniPTEffect(0x4,0);    
 				else
 					UniEffect(UNI_ITEFFECTH,0);
 				UniEffect(UNI_S3MEFFECTD,inf);
@@ -216,7 +218,7 @@ void S3MIT_ProcessCmd(UBYTE cmd,UBYTE inf,unsigned int flags)
 				break;
 			case 0xd: /* Mxx Set Channel Volume */
 				UniEffect(UNI_ITEFFECTM,inf);
-				break;
+				break;       
 			case 0xe: /* Nxy Slide Channel Volume */
 				UniEffect(UNI_ITEFFECTN,inf);
 				break;
@@ -231,7 +233,7 @@ void S3MIT_ProcessCmd(UBYTE cmd,UBYTE inf,unsigned int flags)
 				if(inf && !lo && !(flags & S3MIT_OLDSTYLE))
 					UniWriteByte(1);
 				else
-					UniWriteByte(inf);
+					UniWriteByte(inf); 
 				break;
 			case 0x12: /* Rxy tremolo speed x, depth y */
 				UniEffect(UNI_S3MEFFECTR,inf);
