@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: sloader.c,v 1.1.1.1 2004/06/01 12:16:18 raph Exp $
+  $Id$
 
   Routines for loading samples. The sample loader utilizes the routines
   provided by the "registered" sample loader.
@@ -56,7 +56,7 @@ typedef struct ITPACK {
 BOOL SL_Init(SAMPLOAD* s)
 {
 	if(!sl_buffer)
-		if(!(sl_buffer=_mm_malloc(SLBUFSIZE*sizeof(SWORD)))) return 0;
+		if(!(sl_buffer=MikMod_malloc(SLBUFSIZE*sizeof(SWORD)))) return 0;
 
 	sl_rlength = s->length;
 	if(s->infmt & SF_16BITS) sl_rlength>>=1;
@@ -69,15 +69,15 @@ void SL_Exit(SAMPLOAD *s)
 {
 	if(sl_rlength>0) _mm_fseek(s->reader,sl_rlength,SEEK_CUR);
 	if(sl_buffer) {
-		free(sl_buffer);
+		MikMod_free(sl_buffer);
 		sl_buffer=NULL;
 	}
 }
 
 /* unpack a 8bit IT packed sample */
-static int read_itcompr8(ITPACK* status,MREADER *reader,SWORD *out,UWORD count,UWORD* incnt)
+static BOOL read_itcompr8(ITPACK* status,MREADER *reader,SWORD *sl_buffer,UWORD count,UWORD* incnt)
 {
-	SWORD *dest=out,*end=out+count;
+	SWORD *dest=sl_buffer,*end=sl_buffer+count;
 	UWORD x,y,needbits,havebits,new_count=0;
 	UWORD bits = status->bits;
 	UWORD bufbits = status->bufbits;
@@ -145,13 +145,13 @@ static int read_itcompr8(ITPACK* status,MREADER *reader,SWORD *out,UWORD count,U
 	status->bufbits = bufbits;
 	status->last = last;
 	status->buf = buf;
-	return (dest-out);
+	return (dest-sl_buffer);
 }
 
 /* unpack a 16bit IT packed sample */
-static int read_itcompr16(ITPACK *status,MREADER *reader,SWORD *out,UWORD count,UWORD* incnt)
+static BOOL read_itcompr16(ITPACK *status,MREADER *reader,SWORD *sl_buffer,UWORD count,UWORD* incnt)
 {
-	SWORD *dest=out,*end=out+count;
+	SWORD *dest=sl_buffer,*end=sl_buffer+count;
 	SLONG x,y,needbits,havebits,new_count=0;
 	UWORD bits = status->bits;
 	UWORD bufbits = status->bufbits;
@@ -174,15 +174,15 @@ static int read_itcompr16(ITPACK *status,MREADER *reader,SWORD *out,UWORD count,
 			y=needbits<bufbits?needbits:bufbits;
 			x|=(buf &((1<<y)-1))<<havebits;
 			buf>>=y;
-			bufbits-=y;
-			needbits-=y;
-			havebits+=y;
+			bufbits-=(UWORD)y;
+			needbits-=(UWORD)y;
+			havebits+=(UWORD)y;
 		}
 		if (new_count) {
 			new_count = 0;
 			if (++x >= bits)
 				x++;
-			bits = x;
+			bits = (UWORD)x;
 			continue;
 		}
 		if (bits<7) {
@@ -196,13 +196,13 @@ static int read_itcompr16(ITPACK *status,MREADER *reader,SWORD *out,UWORD count,
 			if ((x>y)&&(x<=y+16)) {
 				if ((x-=y)>=bits)
 					x++;
-				bits = x;
+				bits = (UWORD)x;
 				continue;
 			}
 		}
 		else if (bits<18) {
 			if (x>=0x10000) {
-				bits=x-0x10000+1;
+				bits=(UWORD)(x-0x10000+1);
 				continue;
 			}
 		} else {
@@ -219,10 +219,10 @@ static int read_itcompr16(ITPACK *status,MREADER *reader,SWORD *out,UWORD count,
 	status->bufbits = bufbits;
 	status->last = last;
 	status->buf = buf;
-	return (dest-out);
+	return (dest-sl_buffer);
 }
 
-static int SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor,ULONG length,MREADER* reader,BOOL dither)
+static BOOL SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor,ULONG length,MREADER* reader,BOOL dither)
 {
 	SBYTE *bptr = (SBYTE*)buffer;
 	SWORD *wptr = (SWORD*)buffer;
@@ -230,12 +230,7 @@ static int SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor
 
 	int result,c_block=0;	/* compression bytes until next block */
 	ITPACK status;
-	UWORD incnt = 0;
-
-	status.buf = 0;
-	status.last = 0;
-	status.bufbits = 0;
-	status.bits = 0;
+	UWORD incnt;
 
 	while(length) {
 		stodo=(length<SLBUFSIZE)?length:SLBUFSIZE;
@@ -290,7 +285,7 @@ static int SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor
 				sl_old = sl_buffer[t];
 			}
 
-		if((infmt^outfmt) & SF_SIGNED)
+		if((infmt^outfmt) & SF_SIGNED) 
 			for(t=0;t<stodo;t++)
 				sl_buffer[t]^= 0x8000;
 
@@ -304,7 +299,7 @@ static int SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor
 				scaleval = 0;
 				for(u=scalefactor;u && t<stodo;u--,t++)
 					scaleval+=sl_buffer[t];
-				sl_buffer[idx++]=scaleval/(scalefactor-u);
+				sl_buffer[idx++]=(UWORD)(scaleval/(scalefactor-u));
 				length--;
 			}
 			stodo = idx;
@@ -321,7 +316,7 @@ static int SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor
 				while(t<stodo && length) {
 					avgval=sl_buffer[t++];
 					avgval+=sl_buffer[t++];
-					sl_buffer[idx++]=avgval>>1;
+					sl_buffer[idx++]=(SWORD)(avgval>>1);
 					length-=2;
 				}
 				stodo = idx;
@@ -339,10 +334,10 @@ static int SL_LoadInternal(void* buffer,UWORD infmt,UWORD outfmt,int scalefactor
 	return 0;
 }
 
-int SL_Load(void* buffer,SAMPLOAD *smp,ULONG length)
+BOOL SL_Load(void* buffer,SAMPLOAD *smp,ULONG length)
 {
 	return SL_LoadInternal(buffer,smp->infmt,smp->outfmt,smp->scalefactor,
-				length,smp->reader,0);
+	                       length,smp->reader,0);
 }
 
 /* Registers a sample for loading when SL_LoadSamples() is called. */
@@ -361,7 +356,7 @@ SAMPLOAD* SL_RegisterSample(SAMPLE* s,int type,MREADER* reader)
 		return NULL;
 	
 	/* Allocate and add structure to the END of the list */
-	if(!(news=(SAMPLOAD*)_mm_malloc(sizeof(SAMPLOAD)))) return NULL;
+	if(!(news=(SAMPLOAD*)MikMod_malloc(sizeof(SAMPLOAD)))) return NULL;
 
 	if(cruise) {
 		while(cruise->next) cruise=cruise->next;
@@ -387,7 +382,7 @@ static void FreeSampleList(SAMPLOAD* s)
 	while(s) {
 		old = s;
 		s = s->next;
-		free(old);
+		MikMod_free(old);
 	}
 }
 
@@ -409,9 +404,9 @@ static ULONG SampleTotal(SAMPLOAD* samplist,int type)
 static ULONG RealSpeed(SAMPLOAD *s)
 {
 	return(s->sample->speed/(s->scalefactor?s->scalefactor:1));
-}
+}    
 
-static int DitherSamples(SAMPLOAD* samplist,int type)
+static BOOL DitherSamples(SAMPLOAD* samplist,int type)
 {
 	SAMPLOAD *c2smp=NULL;
 	ULONG maxsize, speed;
@@ -419,7 +414,7 @@ static int DitherSamples(SAMPLOAD* samplist,int type)
 
 	if(!samplist) return 0;
 
-	if((maxsize=MD_SampleSpace(type)*1024))
+	if((maxsize=MD_SampleSpace(type)*1024)) 
 		while(SampleTotal(samplist,type)>maxsize) {
 			/* First Pass - check for any 16 bit samples */
 			s = samplist;
@@ -473,17 +468,17 @@ static int DitherSamples(SAMPLOAD* samplist,int type)
 	return 0;
 }
 
-int SL_LoadSamples(void)
+BOOL SL_LoadSamples(void)
 {
-	int rc;
+	BOOL ok;
 
 	_mm_critical = 0;
 
 	if((!musiclist)&&(!sndfxlist)) return 0;
-	rc=DitherSamples(musiclist,MD_MUSIC)||DitherSamples(sndfxlist,MD_SNDFX);
+	ok=DitherSamples(musiclist,MD_MUSIC)||DitherSamples(sndfxlist,MD_SNDFX);
 	musiclist=sndfxlist=NULL;
 
-	return rc;
+	return ok;
 }
 
 void SL_Sample16to8(SAMPLOAD* s)
@@ -520,4 +515,5 @@ void SL_HalveSample(SAMPLOAD* s,int factor)
 	s->sample->loopend   = s->loopend / s->scalefactor;
 }
 
+
 /* ex:set ts=4: */
