@@ -6,12 +6,12 @@
 	it under the terms of the GNU Library General Public License as
 	published by the Free Software Foundation; either version 2 of
 	the License, or (at your option) any later version.
-
+ 
 	This program is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU Library General Public License for more details.
-
+ 
 	You should have received a copy of the GNU Library General Public
 	License along with this library; if not, write to the Free Software
 	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: virtch_common.c,v 1.1.1.1 2004/06/01 12:16:18 raph Exp $
+  $Id$
 
   Common source parts between the two software mixers.
   This file is probably the ugliest part of libmikmod...
@@ -29,21 +29,17 @@
 
 #ifndef _IN_VIRTCH_
 
-#if defined(HAVE_CONFIG_H)
-#include "config.h"
-#endif
-
 #include "mikmod_internals.h"
 
-extern int   VC1_Init(void);
-extern int   VC2_Init(void);
-static int  (*VC_Init_ptr)(void)=VC1_Init;
+extern BOOL  VC1_Init(void);
+extern BOOL  VC2_Init(void);
+static BOOL (*VC_Init_ptr)(void)=VC1_Init;
 extern void  VC1_Exit(void);
 extern void  VC2_Exit(void);
 static void (*VC_Exit_ptr)(void)=VC1_Exit;
-extern int   VC1_SetNumVoices(void);
-extern int   VC2_SetNumVoices(void);
-static int  (*VC_SetNumVoices_ptr)(void);
+extern BOOL  VC1_SetNumVoices(void);
+extern BOOL  VC2_SetNumVoices(void);
+static BOOL (*VC_SetNumVoices_ptr)(void);
 extern ULONG VC1_SampleSpace(int);
 extern ULONG VC2_SampleSpace(int);
 static ULONG (*VC_SampleSpace_ptr)(int);
@@ -51,9 +47,9 @@ extern ULONG VC1_SampleLength(int,SAMPLE*);
 extern ULONG VC2_SampleLength(int,SAMPLE*);
 static ULONG (*VC_SampleLength_ptr)(int,SAMPLE*);
 
-extern int   VC1_PlayStart(void);
-extern int   VC2_PlayStart(void);
-static int  (*VC_PlayStart_ptr)(void);
+extern BOOL  VC1_PlayStart(void);
+extern BOOL  VC2_PlayStart(void);
+static BOOL (*VC_PlayStart_ptr)(void);
 extern void  VC1_PlayStop(void);
 extern void  VC2_PlayStop(void);
 static void (*VC_PlayStop_ptr)(void);
@@ -107,7 +103,7 @@ extern ULONG VC1_VoiceRealVolume(UBYTE);
 extern ULONG VC2_VoiceRealVolume(UBYTE);
 static ULONG (*VC_VoiceRealVolume_ptr)(UBYTE);
 
-#if defined __STDC__ || defined _MSC_VER
+#if defined __STDC__ || defined _MSC_VER || defined MPW_C
 #define VC_PROC0(suffix) \
 MIKMODAPI void VC_##suffix (void) { VC_##suffix##_ptr(); }
 
@@ -125,9 +121,7 @@ MIKMODAPI void VC_##suffix (typ1 a,typ2 b) { VC_##suffix##_ptr(a,b); }
 
 #define VC_FUNC2(suffix,ret,typ1,typ2) \
 MIKMODAPI ret VC_##suffix (typ1 a,typ2 b) { return VC_##suffix##_ptr(a,b); }
-
 #else
-
 #define VC_PROC0(suffix) \
 MIKMODAPI void VC_/**/suffix (void) { VC_/**/suffix/**/_ptr(); }
 
@@ -147,12 +141,12 @@ MIKMODAPI void VC_/**/suffix (typ1 a,typ2 b) { VC_/**/suffix/**/_ptr(a,b); }
 MIKMODAPI ret VC_/**/suffix (typ1 a,typ2 b) { return VC_/**/suffix/**/_ptr(a,b); }
 #endif
 
-VC_FUNC0(Init,int)
+VC_FUNC0(Init,BOOL)
 VC_PROC0(Exit)
-VC_FUNC0(SetNumVoices,int)
+VC_FUNC0(SetNumVoices,BOOL)
 VC_FUNC1(SampleSpace,ULONG,int)
 VC_FUNC2(SampleLength,ULONG,int,SAMPLE*)
-VC_FUNC0(PlayStart,int)
+VC_FUNC0(PlayStart,BOOL)
 VC_PROC0(PlayStop)
 VC_FUNC2(SampleLoad,SWORD,struct SAMPLOAD*,int)
 VC_PROC1(SampleUnload,SWORD)
@@ -164,16 +158,15 @@ VC_PROC2(VoiceSetFrequency,UBYTE,ULONG)
 VC_FUNC1(VoiceGetFrequency,ULONG,UBYTE)
 VC_PROC2(VoiceSetPanning,UBYTE,ULONG)
 VC_FUNC1(VoiceGetPanning,ULONG,UBYTE)
-
-void VC_VoicePlay(UBYTE a,SWORD b,ULONG c,ULONG d,ULONG e,ULONG f,UWORD g) {
-     VC_VoicePlay_ptr(a,b,c,d,e,f,g);
-}
+		
+void  VC_VoicePlay(UBYTE a,SWORD b,ULONG c,ULONG d,ULONG e,ULONG f,UWORD g)
+{ VC_VoicePlay_ptr(a,b,c,d,e,f,g); }
 
 VC_PROC1(VoiceStop,UBYTE)
 VC_FUNC1(VoiceStopped,BOOL,UBYTE)
 VC_FUNC1(VoiceGetPosition,SLONG,UBYTE)
 VC_FUNC1(VoiceRealVolume,ULONG,UBYTE)
-
+		
 void VC_SetupPointers(void)
 {
 	if (md_mode&DMODE_HQMIXER) {
@@ -232,14 +225,16 @@ void VC_SetupPointers(void)
 
 static ULONG samples2bytes(ULONG samples)
 {
-	if(vc_mode & DMODE_16BITS) samples <<= 1;
+	if(vc_mode & DMODE_FLOAT) samples <<= 2;
+	else if(vc_mode & DMODE_16BITS) samples <<= 1;
 	if(vc_mode & DMODE_STEREO) samples <<= 1;
 	return samples;
 }
 
 static ULONG bytes2samples(ULONG bytes)
 {
-	if(vc_mode & DMODE_16BITS) bytes >>= 1;
+	if(vc_mode & DMODE_FLOAT) bytes >>= 2;
+	else if(vc_mode & DMODE_16BITS) bytes >>= 1;
 	if(vc_mode & DMODE_STEREO) bytes >>= 1;
 	return bytes;
 }
@@ -251,7 +246,9 @@ ULONG VC1_SilenceBytes(SBYTE* buf,ULONG todo)
 	todo=samples2bytes(bytes2samples(todo));
 
 	/* clear the buffer to zero (16 bits signed) or 0x80 (8 bits unsigned) */
-	if(vc_mode & DMODE_16BITS)
+	if(vc_mode & DMODE_FLOAT)
+		memset(buf,0,todo);
+	else if(vc_mode & DMODE_16BITS)
 		memset(buf,0,todo);
 	else
 		memset(buf,0x80,todo);
@@ -277,14 +274,14 @@ ULONG VC1_WriteBytes(SBYTE* buf,ULONG todo)
 
 void VC1_Exit(void)
 {
-	if(vc_tickbuf) free(vc_tickbuf);
-	if(vinf) free(vinf);
-	if(Samples) free(Samples);
+	if(vc_tickbuf) MikMod_free(vc_tickbuf);
+	if(vinf) MikMod_free(vinf);
+	if(Samples) MikMod_free(Samples);
 
 	vc_tickbuf = NULL;
 	vinf = NULL;
 	Samples = NULL;
-
+	
 	VC_SetupPointers();
 }
 
@@ -310,19 +307,19 @@ ULONG VC1_VoiceGetFrequency(UBYTE voice)
 
 void VC1_VoicePlay(UBYTE voice,SWORD handle,ULONG start,ULONG size,ULONG reppos,ULONG repend,UWORD flags)
 {
-	vinf[voice].flags  = flags;
-	vinf[voice].handle = handle;
-	vinf[voice].start  = start;
-	vinf[voice].size   = size;
-	vinf[voice].reppos = reppos;
-	vinf[voice].repend = repend;
-	vinf[voice].kick   = 1;
+	vinf[voice].flags    = flags;
+	vinf[voice].handle   = handle;
+	vinf[voice].start    = start;
+	vinf[voice].size     = size;
+	vinf[voice].reppos   = reppos;
+	vinf[voice].repend   = repend;
+	vinf[voice].kick     = 1;
 }
 
 void VC1_VoiceStop(UBYTE voice)
 {
 	vinf[voice].active = 0;
-}
+}  
 
 BOOL VC1_VoiceStopped(UBYTE voice)
 {
@@ -331,11 +328,11 @@ BOOL VC1_VoiceStopped(UBYTE voice)
 
 SLONG VC1_VoiceGetPosition(UBYTE voice)
 {
-	return(vinf[voice].current>>FRACBITS);
+	return (SLONG)(vinf[voice].current>>FRACBITS);
 }
 
 void VC1_VoiceSetVolume(UBYTE voice,UWORD vol)
-{
+{    
 	/* protect against clicks if volume variation is too high */
 	if(abs((int)vinf[voice].vol-(int)vol)>32)
 		vinf[voice].rampvol=CLICK_BUFFER;
@@ -356,7 +353,7 @@ void VC1_SampleUnload(SWORD handle)
 {
 	if (Samples && (handle < MAXSAMPLEHANDLES)) {
 		if (Samples[handle])
-			free(Samples[handle]);
+			MikMod_free(Samples[handle]);
 		Samples[handle]=NULL;
 	}
 }
@@ -365,7 +362,7 @@ SWORD VC1_SampleLoad(struct SAMPLOAD* sload,int type)
 {
 	SAMPLE *s = sload->sample;
 	int handle;
-	ULONG t, length,loopstart,loopend,looplen;
+	ULONG t, length,loopstart,loopend;
 
 	if(type==MD_HARDWARE) return -1;
 
@@ -377,7 +374,7 @@ SWORD VC1_SampleLoad(struct SAMPLOAD* sload,int type)
 		_mm_errno = MMERR_OUT_OF_HANDLES;
 		return -1;
 	}
-
+	
 	/* Reality check for loop settings */
 	if (s->loopend > s->length)
 		s->loopend = s->length;
@@ -391,7 +388,7 @@ SWORD VC1_SampleLoad(struct SAMPLOAD* sload,int type)
 	SL_SampleSigned(sload);
 	SL_Sample8to16(sload);
 
-	if(!(Samples[handle]=(SWORD*)_mm_malloc((length+20)<<1))) {
+	if(!(Samples[handle]=(SWORD*)MikMod_malloc((length+20)<<1))) {
 		_mm_errno = MMERR_SAMPLE_TOO_BIG;
 		return -1;
 	}
@@ -402,12 +399,11 @@ SWORD VC1_SampleLoad(struct SAMPLOAD* sload,int type)
 
 	/* Unclick sample */
 	if(s->flags & SF_LOOP) {
-		looplen = loopend - loopstart;/* handle short samples */
 		if(s->flags & SF_BIDI)
-			for(t=0;t<16 && t<looplen;t++)
+			for(t=0;t<16;t++)
 				Samples[handle][loopend+t]=Samples[handle][(loopend-t)-1];
 		else
-			for(t=0;t<16 && t<looplen;t++)
+			for(t=0;t<16;t++)
 				Samples[handle][loopend+t]=Samples[handle][t+loopstart];
 	} else
 		for(t=0;t<16;t++)
@@ -435,7 +431,7 @@ ULONG VC1_VoiceRealVolume(UBYTE voice)
 	SWORD *smp;
 	SLONG t;
 
-	t = vinf[voice].current>>FRACBITS;
+	t = (SLONG)(vinf[voice].current>>FRACBITS);
 	if(!vinf[voice].active) return 0;
 
 	s = vinf[voice].handle;
@@ -456,8 +452,9 @@ ULONG VC1_VoiceRealVolume(UBYTE voice)
 	return abs(k-j);
 }
 
-#endif /* _VIRTCH_COMMON_ */
 
-#endif /* _IN_VIRTCH_ */
+#endif
+
+#endif
 
 /* ex:set ts=4: */
